from re import compile as re_compile
from typing import List, Optional, Pattern, Union

from .model import Model


class Plugins:
    _commands = []
    _preprocessors = []

    def __new__(cls) -> tuple:
        commands, preprocessors = cls._commands, cls._preprocessors
        cls._commands, cls._preprocessors = [], []
        return commands, preprocessors

    @classmethod
    def before_command(cls):
        """
        注册plugins预处理器，将在检查所有commands前执行
        """

        def wrap(func: Model.MESSAGE):
            cls._preprocessors.append(func)
            return func

        return wrap

    @classmethod
    def on_command(
        cls,
        command: Optional[Union[List[str], str]] = None,
        regex: Optional[Union[Pattern, str]] = None,
        is_treat: bool = True,
        is_require_at: bool = False,
        is_short_circuit: bool = True,
        is_custom_short_circuit: bool = False,
        is_require_admin: bool = False,
        admin_error_msg: Optional[str] = None,
    ):
        """
        注册plugins指令装饰器，可用于分割式编写指令并注册进机器人

        :param command: 可触发事件的指令列表，与正则regex互斥，优先使用此项
        :param regex: 可触发指令的正则compile实例或正则表达式，与指令表互斥
        :param is_treat: 是否在treated_msg中同时处理指令，如正则将返回.groups()，默认是
        :param is_require_at: 是否要求必须艾特机器人才能触发指令，默认否
        :param is_short_circuit: 如果触发指令成功是否短路不运行后续指令（将根据注册顺序排序指令的短路机制），默认是
        :param is_custom_short_circuit: 如果触发指令成功而返回True则不运行后续指令，与is_short_circuit不能同时存在，默认否
        :param is_require_admin: 是否要求频道主或或管理才可触发指令，默认否
        :param admin_error_msg: 当is_require_admin为True，而触发用户的权限不足时，如此项不为None，返回此消息并短路；否则不进行短路
        """

        def wrap(func: Model.MESSAGE):
            if is_short_circuit and is_custom_short_circuit:
                raise AttributeError(
                    "注意is_short_circuit与is_custom_short_circuit存在冲突，不可同时为True"
                )
            if command:
                if isinstance(command, str):
                    cls._commands.append(
                        {
                            "command": [command],
                            "func": func,
                            "treat": is_treat,
                            "at": is_require_at,
                            "short_circuit": is_short_circuit,
                            "is_custom_short_circuit": is_custom_short_circuit,
                            "admin": is_require_admin,
                            "admin_error_msg": admin_error_msg,
                        }
                    )
                elif isinstance(command, list):
                    cls._commands.append(
                        {
                            "command": command,
                            "func": func,
                            "treat": is_treat,
                            "at": is_require_at,
                            "short_circuit": is_short_circuit,
                            "is_custom_short_circuit": is_custom_short_circuit,
                            "admin": is_require_admin,
                            "admin_error_msg": admin_error_msg,
                        }
                    )
                else:
                    raise TypeError("command参数仅接受str或list类型的指令内容")
            else:
                if isinstance(regex, str):
                    cls._commands.append(
                        {
                            "regex": re_compile(regex),
                            "func": func,
                            "treat": is_treat,
                            "at": is_require_at,
                            "short_circuit": is_short_circuit,
                            "is_custom_short_circuit": is_custom_short_circuit,
                            "admin": is_require_admin,
                            "admin_error_msg": admin_error_msg,
                        }
                    )
                elif isinstance(regex, Pattern):
                    cls._commands.append(
                        {
                            "regex": regex,
                            "func": func,
                            "treat": is_treat,
                            "at": is_require_at,
                            "short_circuit": is_short_circuit,
                            "is_custom_short_circuit": is_custom_short_circuit,
                            "admin": is_require_admin,
                            "admin_error_msg": admin_error_msg,
                        }
                    )
                else:
                    raise TypeError("regex参数仅接受re.compile返回的实例或str类型的正则表达式")
            return func

        return wrap



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
from asyncio import iscoroutinefunction
from copy import deepcopy
from functools import wraps
from inspect import signature, Signature
from json import dumps, loads
from json.decoder import JSONDecodeError
from sys import exc_info
from time import localtime, strftime
from traceback import extract_tb
from typing import BinaryIO, Callable, Optional, Union

from .version import __version__

general_header = {"User-Agent": f"qg-botsdk v{__version__}"}
security_header = {
    "Content-Type": "application/json",
    "charset": "UTF-8",
    "User-Agent": f"qg-botsdk v{__version__}",
}
retry_err_code = (
    101,
    11281,
    11252,
    11263,
    11242,
    11252,
    130000,
    306003,
    306005,
    306006,
    501002,
    501003,
    501004,
    501006,
    501007,
    501011,
    501012,
    620007,
)
msg_t = ("MESSAGE_CREATE", "AT_MESSAGE_CREATE", "DIRECT_MESSAGE_CREATE")
event_t = (
    "GUILD_MEMBER_ADD",
    "GUILD_MEMBER_UPDATE",
    "GUILD_MEMBER_REMOVE",
    "MESSAGE_REACTION_ADD",
    "MESSAGE_REACTION_REMOVE",
    "FORUM_THREAD_CREATE",
    "FORUM_THREAD_UPDATE",
    "FORUM_THREAD_DELETE",
    "FORUM_POST_CREATE",
    "FORUM_POST_DELETE",
    "FORUM_REPLY_CREATE",
    "FORUM_REPLY_DELETE",
    "INTERACTION_CREATE",
)


def template_wrapper(func):
    @wraps(func)
    def wrap(*args):
        try:
            return func(*args)
        except (JSONDecodeError, AttributeError, KeyError):
            return_ = args[0]
            code = (
                return_.status_code
                if hasattr(return_, "status_code")
                else getattr(return_, "status", None)
            )
            trace_id = getattr(return_, "headers", {}).get("X-Tps-Trace-Id")
            return objectize(
                {"data": None, "trace_id": trace_id, "http_code": code, "result": False}
            )

    return wrap


def security_wrapper(func):
    @wraps(func)
    def wrap(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except (JSONDecodeError, KeyError, AttributeError) as e:
            name = func.__name__
            logger = getattr(args[0], "logger", None)
            if name == "__security_check_code":
                if logger:
                    logger.error("无法调用内容安全检测接口（备注：请检查机器人密钥是否正确）")
            else:
                if logger:
                    logger.error(f"调用内容安全检测接口失败，详情：{exception_handler(e)}")
                return False

    return wrap


def exception_handler(error):
    error_info = extract_tb(exc_info()[-1])[-1]
    return '[error:{}] File "{}", line {}, in {}'.format(
        error.__repr__(), *error_info[:3]
    )


def exception_processor(func):
    @wraps(func)
    def wrap(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logger = getattr(args[0], "logger", None)
            if logger:
                logger.error(e.__repr__())
                logger.debug(exception_handler(e))
            else:
                t = strftime("%m-%d %H:%M:%S", localtime())
                print(f"\033[1;31m[{t}] [ERROR]\033[0m {e.__repr__()}")
                print(f"[{t}] [DEBUG] {exception_handler(e)}")

    return wrap


class object_class(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            self.__setattr__(k, v)

    def __getattribute__(self, item):
        if item == "__doc__":
            return object.__getattribute__(self, "__repr__")()
        return object.__getattribute__(self, item)

    def __repr__(self):
        try:
            return dumps(self._data)
        except (TypeError, ValueError):
            return str(self._data)

    @property
    def dict(self):
        return self._data


def _send_msg():  # Cannot direct import from _api_model.py since circular import
    class SendMsg(object_class):
        class data:
            id: str
            channel_id: str
            guild_id: str
            content: str
            timestamp: str
            tts: bool
            mention_everyone: bool
            author: dict
            pinned: bool
            type: int
            flags: int
            seq_in_channel: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return SendMsg


class event_class(object_class):
    def reply(
        self,
        content: Optional[str] = None,
        image: Optional[str] = None,
        file_image: Optional[Union[bytes, BinaryIO, str]] = None,
        message_reference_id: Optional[str] = None,
        ignore_message_reference_error: Optional[bool] = None,
    ) -> _send_msg():
        """
        目前支持reply()发送被动消息的事件类型有:

        GUILD_MEMBER_ADD GUILD_MEMBER_UPDATE GUILD_MEMBER_REMOVE MESSAGE_REACTION_ADD MESSAGE_REACTION_REMOVE
        FORUM_THREAD_CREATE FORUM_THREAD_UPDATE FORUM_THREAD_DELETE FORUM_POST_CREATE FORUM_POST_DELETE
        FORUM_REPLY_CREATE FORUM_REPLY_DELETE INTERACTION_CREATE

        剩余事件的reply()将会转为发送主动消息

        :param content: 消息文本（选填，此项与image至少需要有一个字段，否则无法下发消息）
        :param image: 图片url，不可发送本地图片（选填，此项与msg至少需要有一个字段，否则无法下发消息）
        :param file_image: 本地图片，可选三种方式传参，具体可参阅github中的example_10或帮助文档
        :param message_reference_id: 引用消息的id（选填）
        :param ignore_message_reference_error: 是否忽略获取引用消息详情错误，默认否（选填）
        :return: 返回的.data中为解析后的json数据
        """
        kwargs = locals()
        kwargs.pop("self", None)
        t = getattr(self, "t", None)
        if t in msg_t:
            kwargs["message_id"] = getattr(self, "id", None)
        elif t in event_t:
            kwargs["message_id"] = getattr(self, "event_id", None)
        if "DIRECT_MESSAGE" in t:
            kwargs["guild_id"] = getattr(self, "guild_id", None)
            return getattr(self, "api").send_dm(**kwargs)
        else:
            kwargs["channel_id"] = (
                self.channel_id
                if hasattr(self, "channel_id")
                else getattr(self, "id", None)
            )
            return getattr(self, "api").send_msg(**kwargs)


class async_event_class(object_class):
    async def reply(
        self,
        content: Optional[str] = None,
        image: Optional[str] = None,
        file_image: Optional[Union[bytes, BinaryIO, str]] = None,
        message_reference_id: Optional[str] = None,
        ignore_message_reference_error: Optional[bool] = None,
    ) -> _send_msg():
        """
        目前支持reply()发送被动消息的事件类型有:

        GUILD_MEMBER_ADD GUILD_MEMBER_UPDATE GUILD_MEMBER_REMOVE MESSAGE_REACTION_ADD MESSAGE_REACTION_REMOVE
        FORUM_THREAD_CREATE FORUM_THREAD_UPDATE FORUM_THREAD_DELETE FORUM_POST_CREATE FORUM_POST_DELETE
        FORUM_REPLY_CREATE FORUM_REPLY_DELETE INTERACTION_CREATE

        剩余事件的reply()将会转为发送主动消息

        :param content: 消息文本（选填，此项与image至少需要有一个字段，否则无法下发消息）
        :param image: 图片url，不可发送本地图片（选填，此项与msg至少需要有一个字段，否则无法下发消息）
        :param file_image: 本地图片，可选三种方式传参，具体可参阅github中的example_10或帮助文档
        :param message_reference_id: 引用消息的id（选填）
        :param ignore_message_reference_error: 是否忽略获取引用消息详情错误，默认否（选填）
        :return: 返回的.data中为解析后的json数据
        """
        kwargs = locals()
        kwargs.pop("self", None)
        t = getattr(self, "t", None)
        if t in msg_t:
            kwargs["message_id"] = getattr(self, "id", None)
        elif t in event_t:
            kwargs["message_id"] = getattr(self, "event_id", None)
        if "DIRECT_MESSAGE" in t:
            kwargs["guild_id"] = getattr(self, "guild_id", None)
            return await getattr(self, "api").send_dm(**kwargs)
        else:
            kwargs["channel_id"] = (
                self.channel_id
                if hasattr(self, "channel_id")
                else getattr(self, "id", None)
            )
            return await getattr(self, "api").send_msg(**kwargs)


def objectize(
    data, api=None, is_async=False, is_recursion=False
):  # if api is not None, the event is a resp class
    if isinstance(data, dict):
        _data = data
        if not is_recursion:
            # create a copy that doesn't reference to the original data
            data = deepcopy(data)
        # main func to process data
        for keys, values in data.items():
            if keys.isnumeric():
                return data
            if isinstance(values, dict):
                data[keys] = objectize(values, is_recursion=True)
            elif isinstance(values, list):
                for i, items in enumerate(values):
                    if isinstance(items, dict):
                        data[keys][i] = objectize(items, is_recursion=True)
        data["_data"] = _data
        if api:
            data["api"] = api
            object_data = async_event_class(**data) if is_async else event_class(**data)
        else:
            object_data = object_class(**data)
        return object_data
    else:
        return data


def treat_msg(raw_msg: str, at: str):
    raw_msg = raw_msg if raw_msg.find(at) else raw_msg.replace(at, "", 1).strip()
    if not raw_msg:
        return ""
    if raw_msg[0] == "/":
        raw_msg = raw_msg[1:]
    return (
        raw_msg.replace("&amp;", "&")
        .replace("&lt;", "<")
        .replace("&gt;", ">")
        .replace("\xa0", " ")
        .strip()
    )


def treat_thread(data: dict):
    for items in ("content", "title"):
        try:
            data["d"]["thread_info"][items] = loads(
                data.get("d", {}).get("thread_info", {}).get(items, "{}")
            )
        except JSONDecodeError:
            pass


@template_wrapper
async def http_temp(return_, code: int):
    trace_id = return_.headers.get("X-Tps-Trace-Id")
    real_code = return_.status
    if real_code == code:
        return objectize(
            {"data": None, "trace_id": trace_id, "http_code": real_code, "result": True}
        )
    else:
        return_dict = await return_.json()
        return objectize(
            {
                "data": return_dict,
                "trace_id": trace_id,
                "http_code": real_code,
                "result": False,
            }
        )


@template_wrapper
async def regular_temp(return_):
    trace_id = return_.headers.get("X-Tps-Trace-Id")
    return_dict = await return_.json()
    if isinstance(return_dict, dict) and "code" in return_dict:
        result = False
    else:
        result = True
    return objectize(
        {
            "data": return_dict,
            "trace_id": trace_id,
            "http_code": return_.status,
            "result": result,
        }
    )


@template_wrapper
async def empty_temp(return_):
    trace_id = return_.headers.get("X-Tps-Trace-Id")
    return_dict = await return_.json()
    if not return_dict:
        result = True
        return_dict = None
    else:
        result = False
    return objectize(
        {
            "data": return_dict,
            "trace_id": trace_id,
            "http_code": return_.status,
            "result": result,
        }
    )


def sdk_error_temp(message):
    return objectize(
        {
            "data": {"code": -1, "message": f"这是来自SDK的错误信息：{message}"},
            "trace_id": None,
            "http_code": None,
            "result": False,
        }
    )


def check_func(func, *args, is_async: bool = False):
    sig = signature(func).parameters
    sig_keys = list(sig.keys())
    sig_keys_len = len(sig_keys)
    if sig_keys_len != len(args):
        raise TypeError(f'函数{func.__name__}应包含以下类型的参数：{" ".join(args)}')
    for i in range(sig_keys_len):
        annotation = sig[sig_keys[i]].annotation
        if annotation is not Signature.empty and annotation != args[i]:
            raise TypeError(f"函数{func.__name__}中{sig_keys[i]}参数应为类型：{args[i]}")
    if is_async:
        if not iscoroutinefunction(func):
            raise TypeError(f"函数{func.__name__}应为一个async coroutine函数")
    else:
        if not isinstance(func, Callable) or iscoroutinefunction(func):
            raise TypeError(f"函数{func.__name__}应为一个普通函数")



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
from asyncio import AbstractEventLoop, run_coroutine_threadsafe
from typing import BinaryIO, Dict, List, Optional, Tuple, Union

from . import _api_model
from .async_api import AsyncAPI


class API:
    def __init__(self, api: AsyncAPI, loop: AbstractEventLoop):
        self._api = api
        self._loop = loop

    def security_setup(self, mini_id: str, mini_secret: str):
        self._api._mini_id = mini_id
        self._api._mini_secret = mini_secret

    def security_check(self, content: str) -> bool:
        """
        腾讯小程序侧内容安全检测接口，使用此接口必须填入bot_secret密钥

        :param content: 需要检测的内容
        :return: True或False（bool），代表是否通过安全检测
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.security_check(**_args), self._loop
        )
        return future_.result()

    def get_bot_info(self) -> _api_model.get_bot_info():
        """
        获取机器人详情

        :return:返回的.data中为解析后的json数据
        """
        future_ = run_coroutine_threadsafe(self._api.get_bot_info(), self._loop)
        return future_.result()

    def get_bot_guilds(self) -> _api_model.get_bot_guilds():
        """
        获取机器人所在的所有频道列表

        :return: 返回的.data中为包含所有数据的一个list，列表每个项均为object数据
        """
        future_ = run_coroutine_threadsafe(self._api.get_bot_guilds(), self._loop)
        return future_.result()

    def get_guild_info(self, guild_id: str) -> _api_model.get_guild_info():
        """
        获取频道详情信息

        :param guild_id: 频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_guild_info(**_args), self._loop
        )
        return future_.result()

    def get_guild_channels(self, guild_id: str) -> _api_model.get_guild_channels():
        """
        获取频道的所有子频道列表数据

        :param guild_id: 频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_guild_channels(**_args), self._loop
        )
        return future_.result()

    def get_channels_info(self, channel_id: str) -> _api_model.get_channels_info():
        """
        获取子频道数据

        :param channel_id: 子频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_channels_info(**_args), self._loop
        )
        return future_.result()

    def create_channels(
        self,
        guild_id: str,
        name: str,
        type_: int,
        position: int,
        parent_id: str,
        sub_type: int,
        private_type: int,
        private_user_ids: List[str],
        speak_permission: int,
        application_id: Optional[str] = None,
    ) -> _api_model.create_channels():
        """
        用于在 guild_id 指定的频道下创建一个子频道，一般仅私域机器人可用

        :param guild_id: 频道id
        :param name: 需要创建的子频道名
        :param type_: 需要创建的子频道类型
        :param position: 需要创建的子频道位置
        :param parent_id: 需要创建的子频道所属分组ID
        :param sub_type: 需要创建的子频道子类型
        :param private_type: 需要创建的子频道私密类型
        :param private_user_ids: 需要创建的子频道私密类型成员ID列表
        :param speak_permission: 需要创建的子频道发言权限
        :param application_id: 需要创建的应用类型子频道应用 AppID，仅应用子频道需要该字段
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_channels(**_args), self._loop
        )
        return future_.result()

    def patch_channels(
        self,
        channel_id: str,
        name: Optional[str] = None,
        position: Optional[int] = None,
        parent_id: Optional[str] = None,
        private_type: Optional[int] = None,
        speak_permission: Optional[int] = None,
    ) -> _api_model.patch_channels():
        """
        用于修改 channel_id 指定的子频道的信息，需要修改哪个字段，就传递哪个字段即可

        :param channel_id: 目标子频道ID
        :param name: 子频道名
        :param position: 子频道排序
        :param parent_id: 子频道所属分组id
        :param private_type: 子频道私密类型
        :param speak_permission: 子频道发言权限
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.patch_channels(**_args), self._loop
        )
        return future_.result()

    def delete_channels(self, channel_id) -> _api_model.delete_channels():
        """
        用于删除 channel_id 指定的子频道

        :param channel_id: 子频道id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.delete_channels(**_args), self._loop
        )
        return future_.result()

    def get_guild_members(self, guild_id: str) -> _api_model.get_guild_members():
        """
        用于获取 guild_id 指定的频道中所有成员的详情列表

        :param guild_id: 频道id
        :return: 返回的.data中为包含所有数据的一个list，列表每个项均为object数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_guild_members(**_args), self._loop
        )
        return future_.result()

    def get_role_members(
        self, guild_id: str, role_id: str
    ) -> _api_model.get_role_members():
        """
        用于获取 guild_id 频道中指定 role_id 身份组下所有成员的详情列表

        :param guild_id: 频道id
        :param role_id: 身份组id
        :return: 返回的.data中为包含所有数据的一个list，列表每个项均为object数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_role_members(**_args), self._loop
        )
        return future_.result()

    def get_member_info(
        self, guild_id: str, user_id: str
    ) -> _api_model.get_member_info():
        """
        用于获取 guild_id 指定的频道中 user_id 对应成员的详细信息

        :param guild_id: 频道id
        :param user_id: 成员id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_member_info(**_args), self._loop
        )
        return future_.result()

    def delete_member(
        self,
        guild_id: str,
        user_id: str,
        add_blacklist: bool = False,
        delete_history_msg_days: int = 0,
    ) -> _api_model.delete_member():
        """
        用于删除 guild_id 指定的频道下的成员 user_id

        :param guild_id: 频道ID
        :param user_id: 目标用户ID
        :param add_blacklist: 是否同时添加黑名单
        :param delete_history_msg_days: 用于撤回该成员的消息，可以指定撤回消息的时间范围
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.delete_member(**_args), self._loop)
        return future_.result()

    def get_guild_roles(self, guild_id: str) -> _api_model.get_guild_roles():
        """
        用于获取 guild_id指定的频道下的身份组列表

        :param guild_id: 频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_guild_roles(**_args), self._loop
        )
        return future_.result()

    def create_role(
        self,
        guild_id: str,
        name: Optional[str] = None,
        hoist: Optional[bool] = None,
        color: Optional[Union[str, Tuple[int, int, int]]] = None,
    ) -> _api_model.create_role():
        """
        用于在 guild_id 指定的频道下创建一个身份组

        :param guild_id: 频道id
        :param name: 身份组名（选填)
        :param hoist: 是否在成员列表中单独展示（选填）
        :param color: 身份组颜色，支持输入RGB的三位tuple或HEX的sting颜色（选填)
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.create_role(**_args), self._loop)
        return future_.result()

    def patch_role(
        self,
        guild_id: str,
        role_id: str,
        name: Optional[str] = None,
        hoist: Optional[bool] = None,
        color: Optional[Union[str, Tuple[int, int, int]]] = None,
    ) -> _api_model.patch_role():
        """
        用于修改频道 guild_id 下 role_id 指定的身份组

        :param guild_id: 频道id
        :param role_id: 需要修改的身份组ID
        :param name: 身份组名（选填)
        :param hoist: 是否在成员列表中单独展示（选填）
        :param color: 身份组颜色，支持输入RGB的三位tuple或HEX的sting颜色（选填)
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.patch_role(**_args), self._loop)
        return future_.result()

    def delete_role(self, guild_id: str, role_id: str) -> _api_model.delete_role():
        """
        用于删除频道 guild_id下 role_id 对应的身份组

        :param guild_id: 频道ID
        :param role_id: 需要删除的身份组ID
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.delete_role(**_args), self._loop)
        return future_.result()

    def create_role_member(
        self,
        user_id: str,
        guild_id: str,
        role_id: str,
        channel_id: Optional[str] = None,
    ) -> _api_model.role_members():
        """
        为频道指定成员添加指定身份组

        :param user_id: 目标用户的id
        :param guild_id: 目标频道guild id
        :param role_id: 身份组编号，可从例如get_roles函数获取
        :param channel_id: 如果要增加的身份组ID是5-子频道管理员，需要输入此项来指定具体是哪个子频道
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_role_member(**_args), self._loop
        )
        return future_.result()

    def delete_role_member(
        self,
        user_id: str,
        guild_id: str,
        role_id: str,
        channel_id: Optional[str] = None,
    ) -> _api_model.role_members():
        """
        删除频道指定成员的指定身份组

        :param user_id: 目标用户的id
        :param guild_id: 目标频道guild id
        :param role_id: 身份组编号，可从例如get_roles函数获取
        :param channel_id: 如果要增加的身份组ID是5-子频道管理员，需要输入此项来指定具体是哪个子频道
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.delete_role_member(**_args), self._loop
        )
        return future_.result()

    def get_channel_member_permission(
        self, channel_id: str, user_id: str
    ) -> _api_model.get_channel_member_permission():
        """
        用于获取 子频道 channel_id 下用户 user_id 的权限

        :param channel_id: 子频道id
        :param user_id: 用户id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_channel_member_permission(**_args), self._loop
        )
        return future_.result()

    def put_channel_member_permission(
        self,
        channel_id: str,
        user_id: str,
        add: Optional[str] = None,
        remove: Optional[str] = None,
    ) -> _api_model.put_channel_mr_permission():
        """
        用于修改子频道 channel_id 下用户 user_id 的权限

        :param channel_id: 子频道id
        :param user_id: 用户id
        :param add: 需要添加的权限，string格式，可选：1，2，4，8
        :param remove:需要删除的权限，string格式，可选：1，2，4，8
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.put_channel_member_permission(**_args), self._loop
        )
        return future_.result()

    def get_channel_role_permission(
        self, channel_id: str, role_id: str
    ) -> _api_model.get_channel_role_permission():
        """
        用于获取 子频道 channel_id 下身份组 role_id 的权限

        :param channel_id: 子频道id
        :param role_id: 身份组id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_channel_role_permission(**_args), self._loop
        )
        return future_.result()

    def put_channel_role_permission(
        self,
        channel_id: str,
        role_id: str,
        add: Optional[str] = None,
        remove: Optional[str] = None,
    ) -> _api_model.put_channel_mr_permission():
        """
        用于修改子频道 channel_id 下身份组 role_id 的权限

        :param channel_id: 子频道id
        :param role_id: 身份组id
        :param add: 需要添加的权限，string格式，可选：1，2，4，8
        :param remove:需要删除的权限，string格式，可选：1，2，4，8
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.put_channel_role_permission(**_args), self._loop
        )
        return future_.result()

    def get_message_info(
        self, channel_id: str, message_id: str
    ) -> _api_model.get_message_info():
        """
        用于获取子频道 channel_id 下的消息 message_id 的详情

        :param channel_id: 频道ID
        :param message_id: 目标消息ID
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_message_info(**_args), self._loop
        )
        return future_.result()

    def send_msg(
        self,
        channel_id: str,
        content: Optional[str] = None,
        image: Optional[str] = None,
        file_image: Optional[Union[bytes, BinaryIO, str]] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
        message_reference_id: Optional[str] = None,
        ignore_message_reference_error: Optional[bool] = None,
    ) -> _api_model.send_msg():
        """
        发送普通消息的API

        :param channel_id: 子频道id
        :param content: 消息文本（选填，此项与image至少需要有一个字段，否则无法下发消息）
        :param image: 图片url，不可发送本地图片（选填，此项与msg至少需要有一个字段，否则无法下发消息）
        :param file_image: 本地图片，可选三种方式传参，具体可参阅github中的example_10或帮助文档
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :param message_reference_id: 引用消息的id（选填）
        :param ignore_message_reference_error: 是否忽略获取引用消息详情错误，默认否（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_msg(**_args), self._loop)
        return future_.result()

    def send_embed(
        self,
        channel_id: str,
        title: Optional[str] = None,
        content: Optional[List[str]] = None,
        image: Optional[str] = None,
        prompt: Optional[str] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> _api_model.send_msg():
        """
        发送embed模板消息的API

        :param channel_id: 子频道id
        :param title: 标题文本（选填）
        :param content: 内容文本列表，每一项之间将存在分行（选填）
        :param image: 略缩图url，不可发送本地图片（选填）
        :param prompt: 消息弹窗通知的文本内容（选填）
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_embed(**_args), self._loop)
        return future_.result()

    def send_ark_23(
        self,
        channel_id: str,
        content: List[str],
        link: List[str],
        desc: Optional[str] = None,
        prompt: Optional[str] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> _api_model.send_msg():
        """
        发送ark（id=23）模板消息的API，请注意机器人是否有权限使用此API

        :param channel_id: 子频道id
        :param content: 内容文本列表，每一项之间将存在分行
        :param link: 链接url列表，长度应与内容列一致。将根据位置顺序填充文本超链接，如文本不希望填充链接可使用空文本或None填充位置
        :param desc: 描述文本内容（选填）
        :param prompt: 消息弹窗通知的文本内容（选填）
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_ark_23(**_args), self._loop)
        return future_.result()

    def send_ark_24(
        self,
        channel_id: str,
        title: Optional[str] = None,
        content: Optional[str] = None,
        subtitile: Optional[str] = None,
        link: Optional[str] = None,
        image: Optional[str] = None,
        desc: Optional[str] = None,
        prompt: Optional[str] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> _api_model.send_msg():
        """
        发送ark（id=24）模板消息的API，请注意机器人是否有权限使用此API

        :param channel_id: 子频道id
        :param title: 标题文本（选填）
        :param content: 详情描述文本（选填）
        :param subtitile: 子标题文本（选填）
        :param link: 跳转的链接url（选填）
        :param image: 略缩图url，不可发送本地图片（选填）
        :param desc: 描述文本内容（选填）
        :param prompt: 消息弹窗通知的文本内容（选填）
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_ark_24(**_args), self._loop)
        return future_.result()

    def send_ark_37(
        self,
        channel_id: str,
        title: Optional[str] = None,
        content: Optional[str] = None,
        link: Optional[str] = None,
        image: Optional[str] = None,
        prompt: Optional[str] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> _api_model.send_msg():
        """
        发送ark（id=37）模板消息的API，请注意机器人是否有权限使用此API

        :param channel_id: 子频道id
        :param title: 标题文本（选填）
        :param content: 内容文本（选填）
        :param link: 跳转的链接url（选填）
        :param image: 略缩图url，不可发送本地图片（选填）
        :param prompt: 消息弹窗通知的文本内容（选填）
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_ark_37(**_args), self._loop)
        return future_.result()

    def send_markdown(
        self,
        channel_id: str,
        template_id: Optional[str] = None,
        key_values: Optional[
            Union[
                List[Dict[str, Union[str, List[str]]]], Dict[str, Union[str, List[str]]]
            ]
        ] = None,
        content: Optional[str] = None,
        keyboard_id: Optional[str] = None,
        keyboard_content: Optional[dict] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> _api_model.send_msg():
        """
        发送markdown消息的API，请注意机器人是否有权限使用此API

        :param channel_id: 子频道id
        :param template_id: markdown 模板 id（选填，与content不可同时存在）
        :param key_values: markdown 模版 key values列表，格式为：{key1: value1, key2: value2}（选填，与content不可同时存在）
        :param content: 原生 markdown 内容（选填，与template_id, key, values不可同时存在）
        :param keyboard_id: keyboard 模板 id（选填，与keyboard_content不可同时存在）
        :param keyboard_content: 原生 keyboard 内容（选填，与keyboard_id不可同时存在）
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_markdown(**_args), self._loop)
        return future_.result()

    def delete_msg(
        self, channel_id: str, message_id: str, hidetip: bool = False
    ) -> _api_model.delete_msg():
        """
        撤回消息的API，注意一般情况下仅私域可以使用

        :param channel_id: 子频道id
        :param message_id: 需撤回消息的消息id
        :param hidetip: 是否隐藏提示小灰条，True为隐藏，False为显示（选填）
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.delete_msg(**_args), self._loop)
        return future_.result()

    def get_guild_setting(self, guild_id: str) -> _api_model.get_guild_setting():
        """
        用于获取机器人在频道 guild_id 内的消息频率设置

        :param guild_id: 频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_guild_setting(**_args), self._loop
        )
        return future_.result()

    def create_dm_guild(
        self, target_id: str, guild_id: str
    ) -> _api_model.create_dm_guild():
        """
        当机器人主动跟用户私信时，创建并获取一个虚拟频道id的API

        :param target_id: 目标用户id
        :param guild_id: 机器人跟目标用户所在的频道id
        :return: 返回的.data中为解析后的json数据，注意发送私信仅需要使用guild_id这一项虚拟频道id的数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_dm_guild(**_args), self._loop
        )
        return future_.result()

    def send_dm(
        self,
        guild_id: str,
        content: Optional[str] = None,
        image: Optional[str] = None,
        file_image: Optional[Union[bytes, BinaryIO, str]] = None,
        message_id: Optional[str] = None,
        event_id: Optional[str] = None,
        message_reference_id: Optional[str] = None,
        ignore_message_reference_error: Optional[bool] = None,
    ) -> _api_model.send_msg():
        """
        私信用户的API

        :param guild_id: 虚拟频道id（非子频道id），从用户主动私信机器人的事件、或机器人主动创建私信的API中获取
        :param content: 消息内容文本
        :param image: 图片url，不可发送本地图片（选填，此项与msg至少需要有一个字段，否则无法下发消息）
        :param file_image: 本地图片，可选三种方式传参，具体可参阅github中的example_10或帮助文档
        :param message_id: 消息id（选填）
        :param event_id: 事件id（选填）
        :param message_reference_id: 引用消息的id（选填）
        :param ignore_message_reference_error: 是否忽略获取引用消息详情错误，默认否（选填）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.send_dm(**_args), self._loop)
        return future_.result()

    def delete_dm_msg(
        self, guild_id: str, message_id: str, hidetip: bool = False
    ) -> _api_model.delete_msg():
        """
        用于撤回私信频道 guild_id 中 message_id 指定的私信消息。只能用于撤回机器人自己发送的私信

        :param guild_id: 虚拟频道id（非子频道id），从用户主动私信机器人的事件、或机器人主动创建私信的API中获取
        :param message_id: 需撤回消息的消息id
        :param hidetip: 是否隐藏提示小灰条，True为隐藏，False为显示（选填）
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.delete_dm_msg(**_args), self._loop)
        return future_.result()

    def mute_all_member(
        self,
        guild_id: str,
        mute_end_timestamp: Optional[str],
        mute_seconds: Optional[str],
    ) -> _api_model.mute_member():
        """
        用于将频道的全体成员（非管理员）禁言

        :param guild_id: 频道id
        :param mute_end_timestamp: 禁言到期时间戳，绝对时间戳，单位：秒（与 mute_seconds 字段同时赋值的话，以该字段为准）
        :param mute_seconds: 禁言多少秒（两个字段二选一，默认以 mute_end_timestamp 为准）
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.mute_all_member(**_args), self._loop
        )
        return future_.result()

    def mute_member(
        self,
        guild_id: str,
        user_id: str,
        mute_end_timestamp: Optional[str],
        mute_seconds: Optional[str],
    ) -> _api_model.mute_member():
        """
        用于禁言频道 guild_id 下的成员 user_id

        :param guild_id: 频道id
        :param user_id: 目标成员的用户ID
        :param mute_end_timestamp: 禁言到期时间戳，绝对时间戳，单位：秒（与 mute_seconds 字段同时赋值的话，以该字段为准）
        :param mute_seconds: 禁言多少秒（两个字段二选一，默认以 mute_end_timestamp 为准）
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.mute_member(**_args), self._loop)
        return future_.result()

    def mute_members(
        self,
        guild_id: str,
        user_id: List[str],
        mute_end_timestamp: Optional[str],
        mute_seconds: Optional[str],
    ) -> _api_model.mute_members():
        """
        用于将频道的指定批量成员（非管理员）禁言

        :param guild_id: 频道id
        :param user_id: 目标成员的用户ID列表
        :param mute_end_timestamp: 禁言到期时间戳，绝对时间戳，单位：秒（与 mute_seconds 字段同时赋值的话，以该字段为准）
        :param mute_seconds: 禁言多少秒（两个字段二选一，默认以 mute_end_timestamp 为准）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.mute_members(**_args), self._loop)
        return future_.result()

    def create_announce(
        self,
        guild_id,
        channel_id: Optional[str] = None,
        message_id: Optional[str] = None,
        announces_type: Optional[int] = None,
        recommend_channels_id: Optional[List[str]] = None,
        recommend_channels_introduce: Optional[List[str]] = None,
    ) -> _api_model.create_announce():
        """
        用于创建频道全局公告，公告类型分为 消息类型的频道公告 和 推荐子频道类型的频道公告

        :param guild_id: 频道id
        :param channel_id: 子频道id，message_id 有值则为必填
        :param message_id: 消息id，此项有值则优选将某条消息设置为成员公告
        :param announces_type: 公告类别 0：成员公告，1：欢迎公告，默认为成员公告
        :param recommend_channels_id: 推荐子频道id列表，会一次全部替换推荐子频道列表
        :param recommend_channels_introduce: 推荐子频道推荐语列表，列表长度应与recommend_channels_id一致
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_announce(**_args), self._loop
        )
        return future_.result()

    def delete_announce(
        self, guild_id: str, message_id: str = "all"
    ) -> _api_model.delete_announce():
        """
        用于删除频道 guild_id 下指定 message_id 的全局公告

        :param guild_id: 频道id
        :param message_id: message_id有值时会校验message_id合法性；若不校验，请将message_id设置为all（默认为all）
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.delete_announce(**_args), self._loop
        )
        return future_.result()

    def create_pinmsg(self, channel_id: str, message_id: str) -> _api_model.pinmsg():
        """
        用于添加子频道 channel_id 内的精华消息

        :param channel_id: 子频道id
        :param message_id: 目标消息id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.create_pinmsg(**_args), self._loop)
        return future_.result()

    def delete_pinmsg(
        self, channel_id: str, message_id: str
    ) -> _api_model.delete_pinmsg():
        """
        用于删除子频道 channel_id 下指定 message_id 的精华消息

        :param channel_id: 子频道id
        :param message_id: 目标消息id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.delete_pinmsg(**_args), self._loop)
        return future_.result()

    def get_pinmsg(self, channel_id: str) -> _api_model.pinmsg():
        """
        用于获取子频道 channel_id 内的精华消息

        :param channel_id: 子频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.get_pinmsg(**_args), self._loop)
        return future_.result()

    def get_schedules(
        self, channel_id: str, since: Optional[int] = None
    ) -> _api_model.get_schedules():
        """
        用于获取channel_id指定的子频道中当天的日程列表

        :param channel_id: 日程子频道id
        :param since: 起始时间戳(ms)
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.get_schedules(**_args), self._loop)
        return future_.result()

    def get_schedule_info(
        self, channel_id: str, schedule_id: str
    ) -> _api_model.schedule_info():
        """
        获取日程子频道 channel_id 下 schedule_id 指定的的日程的详情

        :param channel_id: 日程子频道id
        :param schedule_id: 日程id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_schedule_info(**_args), self._loop
        )
        return future_.result()

    def create_schedule(
        self,
        channel_id: str,
        schedule_name: str,
        start_timestamp: str,
        end_timestamp: str,
        jump_channel_id: str,
        remind_type: str,
    ) -> _api_model.schedule_info():
        """
        用于在 channel_id 指定的日程子频道下创建一个日程

        :param channel_id: 日程子频道id
        :param schedule_name: 日程名称
        :param start_timestamp: 日程开始时间戳(ms)
        :param end_timestamp: 日程结束时间戳(ms)
        :param jump_channel_id: 日程开始时跳转到的子频道id
        :param remind_type: 日程提醒类型
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_schedule(**_args), self._loop
        )
        return future_.result()

    def patch_schedule(
        self,
        channel_id: str,
        schedule_id: str,
        schedule_name: str,
        start_timestamp: str,
        end_timestamp: str,
        jump_channel_id: str,
        remind_type: str,
    ) -> _api_model.schedule_info():
        """
        用于修改日程子频道 channel_id 下 schedule_id 指定的日程的详情

        :param channel_id: 日程子频道id
        :param schedule_id: 日程id
        :param schedule_name: 日程名称
        :param start_timestamp: 日程开始时间戳(ms)
        :param end_timestamp: 日程结束时间戳(ms)
        :param jump_channel_id: 日程开始时跳转到的子频道id
        :param remind_type: 日程提醒类型
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.patch_schedule(**_args), self._loop
        )
        return future_.result()

    def delete_schedule(
        self, channel_id: str, schedule_id: str
    ) -> _api_model.delete_schedule():
        """
        用于删除日程子频道 channel_id 下 schedule_id 指定的日程

        :param channel_id: 日程子频道id
        :param schedule_id: 日程id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.delete_schedule(**_args), self._loop
        )
        return future_.result()

    def create_reaction(
        self, channel_id: str, message_id: str, type_: str, id_: str
    ) -> _api_model.reactions():
        """
        对message_id指定的消息进行表情表态

        :param channel_id: 子频道id
        :param message_id: 目标消息id
        :param type_: 表情类型
        :param id_: 表情id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_reaction(**_args), self._loop
        )
        return future_.result()

    def delete_reaction(
        self, channel_id: str, message_id: str, type_: str, id_: str
    ) -> _api_model.reactions():
        """
        删除自己对message_id指定消息的表情表态

        :param channel_id: 子频道id
        :param message_id: 目标消息id
        :param type_: 表情类型
        :param id_: 表情id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.delete_reaction(**_args), self._loop
        )
        return future_.result()

    def get_reaction_users(
        self, channel_id: str, message_id: str, type_: str, id_: str
    ) -> _api_model.get_reaction_users():
        """
        拉取对消息 message_id 指定表情表态的用户列表

        :param channel_id: 子频道id
        :param message_id: 目标消息id
        :param type_: 表情类型
        :param id_: 表情id
        :return: 返回的.data中为解析后的json数据列表
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_reaction_users(**_args), self._loop
        )
        return future_.result()

    def control_audio(
        self,
        channel_id: str,
        status: int,
        audio_url: Optional[str] = None,
        text: Optional[str] = None,
    ) -> _api_model.audio():
        """
        用于控制子频道 channel_id 下的音频

        :param channel_id: 子频道id
        :param status: 播放状态
        :param audio_url: 音频数据的url，可选，status为0时传
        :param text: 状态文本（比如：简单爱-周杰伦），可选，status为0时传，其他操作不传
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.control_audio(**_args), self._loop)
        return future_.result()

    def bot_on_mic(self, channel_id: str) -> _api_model.audio():
        """
        机器人在 channel_id 对应的语音子频道上麦

        :param channel_id: 子频道id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.bot_on_mic(**_args), self._loop)
        return future_.result()

    def bot_off_mic(self, channel_id: str) -> _api_model.audio():
        """
        机器人在 channel_id 对应的语音子频道下麦

        :param channel_id: 子频道id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.bot_off_mic(**_args), self._loop)
        return future_.result()

    def get_threads(self, channel_id: str) -> _api_model.get_threads():
        """
        获取子频道下的帖子列表

        :param channel_id: 目标论坛子频道id
        :return: 返回的.data中为解析后的json数据列表
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.get_threads(**_args), self._loop)
        return future_.result()

    def get_thread_info(
        self, channel_id: str, thread_id: str
    ) -> _api_model.get_thread_info():
        """
        获取子频道下的帖子详情

        :param channel_id: 目标论坛子频道id
        :param thread_id: 帖子id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_thread_info(**_args), self._loop
        )
        return future_.result()

    def create_thread(
        self, channel_id: str, title: str, content: str, format_: int
    ) -> _api_model.create_thread():
        """
        创建帖子，创建成功后，返回创建成功的任务ID

        :param channel_id: 目标论坛子频道id
        :param title: 帖子标题
        :param content: 帖子内容（具体格式根据format_判断）
        :param format_: 帖子文本格式（1：普通文本、2：HTML、3：Markdown、4：Json）
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.create_thread(**_args), self._loop)
        return future_.result()

    def delete_thread(
        self, channel_id: str, thread_id: str
    ) -> _api_model.delete_thread():
        """
        删除指定子频道下的某个帖子

        :param channel_id: 目标论坛子频道id
        :param thread_id: 帖子id
        :return: 返回的.result显示是否成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(self._api.delete_thread(**_args), self._loop)
        return future_.result()

    def get_guild_permissions(
        self, guild_id: str
    ) -> _api_model.get_guild_permissions():
        """
        获取机器人在频道 guild_id 内可以使用的权限列表

        :param guild_id: 频道id
        :return: 返回的.data中为解析后的json数据
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.get_guild_permissions(**_args), self._loop
        )
        return future_.result()

    def create_permission_demand(
        self, guild_id: str, channel_id: str, api: str, desc: Optional[str]
    ) -> _api_model.create_permission_demand():
        """
        发送频道API接口权限授权链接到频道

        :param guild_id: 频道id
        :param channel_id: 子频道id
        :param api: 需求权限的API在sdk的名字
        :param desc: 机器人申请对应的API接口权限后可以使用功能的描述
        :return: 返回成功或不成功
        """
        _args = locals()
        _args.pop("self")
        future_ = run_coroutine_threadsafe(
            self._api.create_permission_demand(**_args), self._loop
        )
        return future_.result()



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
from functools import wraps
from logging import FileHandler, Formatter, StreamHandler, getLogger
from os import PathLike, getcwd, makedirs, sep
from os.path import exists, isdir, join
from re import split as re_split
from time import localtime, strftime
from typing import List, Union

try:
    from colorama import init as color_init

    color_init(strip=False)
except (ImportError, ModuleNotFoundError):
    from os import system

    system("")


def _log_wrapper(func):
    @wraps(func)
    def wrap(self, msg):
        str_time = strftime("%m-%d", localtime())
        if str_time != self._previous_time:
            self._logger.removeHandler(self._logh)
            self._new_logh(str_time)
        try:
            func(self, msg)
        except UnicodeDecodeError:
            pass

    return wrap


class Logger:
    def __init__(
        self,
        bot_app_id: str,
        file_path: Union[str, PathLike] = None,
        disable_logger: List[str] = None,
    ):
        """
        用作logging输出的类，支持不同level的颜色log输出，可自定义格式

        :param bot_app_id: 机器人ID
        :param file_path: 可选填，自定义log文件输出路径，不填默认为：当前目录/log/[bot_app_id]/
        :param disable_logger: 可选填，需要disable的logger名称列表，不填默认无
        """
        getLogger("asyncio").disabled = True
        if disable_logger:
            for items in disable_logger:
                getLogger(items).disabled = True
        self._bot_app_id = bot_app_id
        self._logger = getLogger(__name__)
        self._logger.setLevel("DEBUG")
        self._format = "[%(asctime)s] [%(levelname)s] %(message)s"
        self._date_format = "%m-%d %H:%M:%S"
        self._cmdh = StreamHandler()
        self._cmdh.setFormatter(self._Stream_Formatter())
        self._cmdh.setLevel("INFO")
        self.file_path = file_path
        if file_path is not None:
            file_path = re_split(r"[\\|/]", file_path)
            if file_path[0] == ".":
                self.file_path = join(*file_path)
            else:
                file_path[0] += sep
                self.file_path = join(sep, *file_path)
        else:
            self.file_path = join(getcwd(), "log", self._bot_app_id)
        if not exists(self.file_path):
            makedirs(self.file_path)
        assert isdir(self.file_path), "自定义Log输出路径必须为一个directory资料夹"
        self._logh = None
        self._new_logh(strftime("%m-%d", localtime()))
        self._logger.addHandler(self._cmdh)
        self._previous_time = strftime("%m-%d", localtime())

    class _Stream_Formatter(Formatter):
        FORMATS = {
            20: "\033[1;32m[%(asctime)s] [%(levelname)s]\033[0m %(message)s",
            30: "\033[1;33m[%(asctime)s] [%(levelname)s]\033[0m %(message)s",
            40: "\033[1;31m[%(asctime)s] [%(levelname)s]\033[0m %(message)s",
        }

        def __init__(self, formats: dict = None, date_format: str = None):
            super().__init__()
            if formats is not None:
                self.FORMATS[20] = formats.get(20, None) or self.FORMATS[20]
                self.FORMATS[30] = formats.get(30, None) or self.FORMATS[30]
                self.FORMATS[40] = formats.get(40, None) or self.FORMATS[40]
            self._date_format = date_format or "%m-%d %H:%M:%S"

        def format(self, record) -> str:
            log_fmt = self.FORMATS.get(record.levelno)
            formatter = Formatter(log_fmt, self._date_format)
            return formatter.format(record)

    def setLevel(self, level: Union[int, str]):
        """
        用于设置logger的level，同logging.logger.setLevel()

        `在docs.python.org查阅更多资讯
        <https://docs.python.org/zh-cn/3.10/library/logging.html#logging.Logger.setLevel>`_

        :param level: level必须是int或str
        """
        self._logger.setLevel(level)

    def set_formatter(
        self,
        debug_format: str = None,
        info_format: str = None,
        warning_format: str = None,
        error_format: str = None,
        date_format: str = None,
    ):
        """
        用于更改logger日志的输出格式

        - 更多关于格式的相关信息可参阅：
        - https://docs.python.org/zh-cn/3/library/logging.html#logging.Formatter
        - https://docs.python.org/zh-cn/3.6/library/logging.html#logrecord-attributes

        :param debug_format: debug层级的格式，此格式会输出到保存的日志文件中
        :param info_format: info层级StreamHandler的格式，此格式会输出到python console中显示
        :param warning_format: warning层级StreamHandler的格式，此格式会输出到python console中显示
        :param error_format: error层级StreamHandler的格式，此格式会输出到python console中显示
        :param date_format: 全局所有层级的日期格式
        """
        self._date_format = date_format or self._date_format
        self._format = debug_format or self._format
        formats = {20: info_format, 30: warning_format, 40: error_format}
        self._cmdh.setFormatter(self._Stream_Formatter(formats, date_format))
        self._logh.setFormatter(Formatter(self._format, self._date_format))

    @staticmethod
    def disable_logger(loggers: Union[str, List[str]]):
        """
        用于disable禁用logger

        :param loggers: 需要禁用的logger名称或名称列
        """
        if isinstance(loggers, list):
            for items in loggers:
                getLogger(items).disabled = True
        else:
            getLogger(loggers).disabled = True

    def _new_logh(self, str_time):
        self._logh = FileHandler(
            join(self.file_path, f"{str_time}.log"), encoding="utf-8"
        )
        self._logh.setFormatter(Formatter(self._format, self._date_format))
        self._logger.addHandler(self._logh)

    @_log_wrapper
    def debug(self, msg):
        self._logger.debug(msg)

    @_log_wrapper
    def info(self, msg):
        self._logger.info(msg)

    @_log_wrapper
    def warning(self, msg):
        self._logger.warning(msg)

    @_log_wrapper
    def error(self, msg):
        self._logger.error(msg)



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
from asyncio import Lock as ALock
from asyncio import get_event_loop, new_event_loop, sleep
from os import getpid
from os.path import exists
from re import Pattern
from threading import Lock as TLock
from time import sleep as t_sleep
from typing import Any, Callable, List, Optional, Union

from ._api_model import robot_model
from ._utils import check_func, exception_processor
from .api import API
from .async_api import AsyncAPI
from .http import Session
from .logger import Logger
from .model import Model
from .plugins import Plugins
from .qg_bot_ws import BotWs as _BotWs
from .version import __version__

pid = getpid()
print(f"本次程序进程ID：{pid} | SDK版本：{__version__} | 即将开始运行机器人……")
t_sleep(0.5)


class BOT:
    def __init__(
        self,
        bot_id: str,
        bot_token: str,
        bot_secret: Optional[str] = None,
        is_private: bool = False,
        is_sandbox: bool = False,
        no_permission_warning: bool = True,
        is_async: bool = False,
        is_retry: bool = True,
        is_log_error: bool = True,
        shard_no: int = 0,
        total_shard: int = 1,
        max_workers: int = 32,
        api_max_concurrency: int = 0,
    ):
        """
        机器人主体，输入BotAppID和密钥，并绑定函数后即可快速使用

        :param bot_id: 机器人平台后台BotAppID（开发者ID）项，必填
        :param bot_token: 机器人平台后台机器人令牌项，必填
        :param bot_secret: 机器人平台后台机器人密钥项（已废弃）
        :param is_private: 机器人是否为私域机器人，默认False
        :param is_sandbox: 是否开启沙箱环境，默认False
        :param no_permission_warning: 是否开启当机器人获取疑似权限不足的事件时的警告提示，默认开启
        :param is_async: 使用同步api还是异步api，默认False（使用同步）
        :param is_retry: 使用api时，如遇可重试的错误码是否自动进行重试，默认开启
        :param is_log_error: 使用api时，如返回的结果为不成功，可自动log输出报错信息，默认开启
        :param shard_no: 当前分片数，如不熟悉相关配置请不要轻易改动此项，默认0
        :param total_shard: 最大分片数，如不熟悉相关配置请不要轻易改动此项，默认1
        :param max_workers: 在同步模式下，允许同时运行的最大线程数，默认32
        :param api_max_concurrency: API允许的最大并发数，超过此并发数将进入队列，如此数值<=0代表不开启任何队列
        """
        self.logger = Logger(bot_id)
        self.bot_id = bot_id
        self.bot_token = bot_token
        if bot_secret:
            raise DeprecationWarning(
                "bot_secret已被废弃，如需使用安全接口，请通过security_setup()绑定小程序ID和secret"
            )
        self.is_private = is_private
        self.bot_url = (
            r"https://sandbox.api.sgroup.qq.com"
            if is_sandbox
            else r"https://api.sgroup.qq.com"
        )
        if not bot_id or not bot_token:
            raise type("IdTokenMissing", (Exception,), {})(
                "你还没有输入 bot_id 和 bot_token，无法连接使用机器人\n如尚未有相关票据，"
                "请参阅 https://qg-botsdk.readthedocs.io/zh_CN/latest/quick_start 了解相关详情"
            )
        try:
            self._loop = get_event_loop()
        except RuntimeError:
            self._loop = new_event_loop()
        self._intents = 0
        self._shard_no = shard_no
        self._total_shard = total_shard
        self._bot_class = None
        self._on_delete_function = None
        self._on_msg_function = None
        self._on_dm_function = None
        self._on_guild_event_function = None
        self._on_channel_event_function = None
        self._on_guild_member_function = None
        self._on_reaction_function = None
        self._on_interaction_function = None
        self._on_audit_function = None
        self._on_forum_function = None
        self._on_open_forum_function = None
        self._on_audio_function = None
        self._on_live_channel_member_function = None
        self._repeat_function = None
        self._on_start_function = None
        self.is_filter_self = True
        self.check_interval = 10
        self.__running = False
        self.auth = f"Bot {bot_id}.{bot_token}"
        self.bot_headers = {"Authorization": self.auth}
        self._session = Session(
            self._loop,
            is_retry,
            is_log_error,
            self.logger,
            api_max_concurrency,
            headers=self.bot_headers,
        )
        self.msg_treat = True
        self.dm_treat = False
        self.no_permission_warning = no_permission_warning
        self.max_workers = max_workers
        self.is_async = is_async
        self.api: Union[AsyncAPI, API] = AsyncAPI(
            self.bot_url,
            self._session,
            self.logger,
            self._check_warning,
        )
        if not is_async:
            self.lock = TLock()
            self.api = API(self.api, self._loop)
        else:
            self.lock = ALock()

    def __repr__(self):
        return f"<qg_botsdk.BOT object [id: {self.bot_id}, token: {self.bot_token}]>"

    @property
    def robot(self) -> robot_model():
        return self._bot_class.robot

    @property
    def running(self):
        return self.__running

    @property
    def loop(self):
        return self._loop

    @exception_processor
    def __time_event_run(self):
        if self.is_async:
            self._loop.create_task(self._repeat_function())
        else:
            self._repeat_function()

    async def __time_event_check(self):
        while self.__running:
            await sleep(self.check_interval)
            self.__time_event_run()

    def __register_msg_intents(self, is_log=True):
        if not self._intents >> 9 & 1 and not self._intents >> 30 & 1:
            if self.is_private:
                self._intents = self._intents | 1 << 9
                if is_log:
                    self.logger.info("消息（所有消息）接收函数订阅成功")
            else:
                self._intents = self._intents | 1 << 30
                if is_log:
                    self.logger.info("消息（艾特消息）接收函数订阅成功")

    def _get_plugins(self):
        commands, preprocessors = Plugins()
        if commands:
            for items in commands:
                check_func(items["func"], Model.MESSAGE, is_async=self.is_async)
            self.__register_msg_intents()
        for func in preprocessors:
            check_func(func, Model.MESSAGE, is_async=self.is_async)
        return commands, preprocessors

    @staticmethod
    def load_plugins(path_to_plugins: str):
        """
        用于加载插件的.py程序

        :param path_to_plugins: 指向相应.py插件文件的相对或绝对路径
        :return:
        """
        if not exists(path_to_plugins):
            raise ModuleNotFoundError(f"指向plugin的路径 [{path_to_plugins}] 并不存在")
        try:
            with open(path_to_plugins, "r", encoding="utf-8") as f:
                exec(f.read())
        except Exception:
            raise ImportError(f"plugin [{path_to_plugins}] 导入失败")

    @staticmethod
    def before_command():
        """
        注册预处理器，将在检查所有commands前执行
        """

        def wrap(func: Model.MESSAGE):
            Plugins.before_command()(func)
            return func

        return wrap

    @staticmethod
    def on_command(
        command: Optional[Union[List[str], str]] = None,
        regex: Optional[Union[Pattern, str]] = None,
        is_treat: bool = True,
        is_require_at: bool = False,
        is_short_circuit: bool = True,
        is_custom_short_circuit: bool = False,
        is_require_admin: bool = False,
        admin_error_msg: Optional[str] = None,
    ):
        """
        指令装饰器。用于快速注册消息事件，当连同bind_msg使用时，如没有触发短路，bind_msg注册的函数将在最后被调用

        :param command: 可触发事件的指令列表，与正则regex互斥，优先使用此项
        :param regex: 可触发指令的正则compile实例或正则表达式，与指令表互斥
        :param is_treat: 是否在treated_msg中同时处理指令，如正则将返回.groups()，默认是
        :param is_require_at: 是否要求必须艾特机器人才能触发指令，默认否
        :param is_short_circuit: 如果触发指令成功是否短路不运行后续指令（将根据注册顺序排序指令的短路机制），默认是
        :param is_custom_short_circuit: 如果触发指令成功而返回True则不运行后续指令，与is_short_circuit不能同时存在，默认否
        :param is_require_admin: 是否要求频道主或或管理才可触发指令，默认否
        :param admin_error_msg: 当is_require_admin为True，而触发用户的权限不足时，如此项不为None，返回此消息并短路；否则不进行短路
        """

        def wrap(func: Model.MESSAGE):
            Plugins.on_command(
                command,
                regex,
                is_treat,
                is_require_at,
                is_short_circuit,
                is_custom_short_circuit,
                is_require_admin,
                admin_error_msg,
            )(func)
            return func

        return wrap

    def bind_msg(
        self,
        on_msg_function: Callable[[Model.MESSAGE], Any] = None,
        treated_data: bool = True,
        all_msg: bool = None,
    ):
        """
        用作绑定接收消息的函数，将根据机器人是否公域自动判断接收艾特或所有消息

        :param on_msg_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        :param treated_data: 是否返回经转义处理的文本，如是则会在返回的Object中添加一个treated_msg的子类，默认True
        :param all_msg: 是否无视公私域限制，强制开启全部消息接收，默认None（不判断此项参数）
        """

        def wraps(func):
            check_func(func, Model.MESSAGE, is_async=self.is_async)
            self._on_msg_function = func
            if not treated_data:
                self.msg_treat = False
            if all_msg is None:
                self.__register_msg_intents()
            elif all_msg:
                self._intents = self._intents | 1 << 9
                self.logger.info("消息（所有消息）接收函数订阅成功")
            else:
                self._intents = self._intents | 1 << 30
                self.logger.info("消息（艾特消息）接收函数订阅成功")

        if not on_msg_function:
            return wraps
        wraps(on_msg_function)

    def bind_dm(
        self,
        on_dm_function: Callable[[Model.DIRECT_MESSAGE], Any] = None,
        treated_data: bool = True,
    ):
        """
        用作绑定接收私信消息的函数

        :param on_dm_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        :param treated_data: 是否返回经转义处理的文本，如是则会在返回的Object中添加一个treated_msg的子类，默认True
        """

        def wraps(func):
            check_func(func, Model.DIRECT_MESSAGE, is_async=self.is_async)
            self._on_dm_function = func
            self._intents = self._intents | 1 << 12
            if treated_data:
                self.dm_treat = True
            self.logger.info("私信接收函数订阅成功")

        if not on_dm_function:
            return wraps
        wraps(on_dm_function)

    def bind_msg_delete(
        self,
        on_delete_function: Callable[[Model.MESSAGE_DELETE], Any] = None,
        is_filter_self: bool = True,
    ):
        """
        用作绑定接收消息撤回事件的函数，注册时将自动根据公域私域注册艾特或全部消息，但不会主动注册私信事件

        :param on_delete_function:类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        :param is_filter_self: 是否过滤用户自行撤回的消息，只接受管理撤回事件
        """

        def wraps(func):
            check_func(func, Model.MESSAGE_DELETE, is_async=self.is_async)
            self._on_delete_function = func
            self.is_filter_self = is_filter_self
            self.__register_msg_intents(False)
            self.logger.info("撤回事件订阅成功")

        if not on_delete_function:
            return wraps
        wraps(on_delete_function)

    def bind_guild_event(
        self, on_guild_event_function: Callable[[Model.GUILDS], Any] = None
    ):
        """
        用作绑定接收频道信息的函数

        :param on_guild_event_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.GUILDS, is_async=self.is_async)
            self._on_guild_event_function = func
            self._intents = self._intents | 1
            self.logger.info("频道事件订阅成功")

        if not on_guild_event_function:
            return wraps
        wraps(on_guild_event_function)

    def bind_channel_event(
        self, on_channel_event_function: Callable[[Model.CHANNELS], Any] = None
    ):
        """
        用作绑定接收子频道信息的函数

        :param on_channel_event_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.CHANNELS, is_async=self.is_async)
            self._on_channel_event_function = func
            self._intents = self._intents | 1
            self.logger.info("子频道事件订阅成功")

        if not on_channel_event_function:
            return wraps
        wraps(on_channel_event_function)

    def bind_guild_member(
        self, on_guild_member_function: Callable[[Model.GUILD_MEMBERS], Any] = None
    ):
        """
        用作绑定接收频道成员信息的函数

        :param on_guild_member_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.GUILD_MEMBERS, is_async=self.is_async)
            self._on_guild_member_function = func
            self._intents = self._intents | 1 << 1
            self.logger.info("频道成员事件订阅成功")

        if not on_guild_member_function:
            return wraps
        wraps(on_guild_member_function)

    def bind_reaction(
        self, on_reaction_function: Callable[[Model.REACTION], Any] = None
    ):
        """
        用作绑定接收表情表态信息的函数

        :param on_reaction_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.REACTION, is_async=self.is_async)
            self._on_reaction_function = func
            self._intents = self._intents | 1 << 10
            self.logger.info("表情表态事件订阅成功")

        if not on_reaction_function:
            return wraps
        wraps(on_reaction_function)

    def bind_interaction(
        self, on_interaction_function: Callable[[Model.INTERACTION], Any] = None
    ):
        """
        用作绑定接收互动事件的函数

        :param on_interaction_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.INTERACTION, is_async=self.is_async)
            self._on_interaction_function = func
            self._intents = self._intents | 1 << 26
            self.logger.info("互动事件订阅成功")

        if not on_interaction_function:
            return wraps
        wraps(on_interaction_function)

    def bind_audit(
        self, on_audit_function: Callable[[Model.MESSAGE_AUDIT], Any] = None
    ):
        """
        用作绑定接收审核事件的函数

        :param on_audit_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.MESSAGE_AUDIT, is_async=self.is_async)
            self._on_audit_function = func
            self._intents = self._intents | 1 << 27
            self.logger.info("审核事件订阅成功")

        if not on_audit_function:
            return wraps
        wraps(on_audit_function)

    def bind_forum(self, on_forum_function: Callable[[Model.FORUMS_EVENT], Any] = None):
        """
        用作绑定接收论坛事件的函数，一般仅私域机器人能注册此事件

        .. note::
            当前仅可以接收FORUM_THREAD_CREATE、FORUM_THREAD_UPDATE、FORUM_THREAD_DELETE三个事件

        :param on_forum_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.FORUMS_EVENT, is_async=self.is_async)
            self._on_forum_function = func
            self._intents = self._intents | 1 << 28
            self.logger.info("论坛事件订阅成功")
            if not self.is_private and self.no_permission_warning:
                self.logger.warning("请注意，一般公域机器人并不能注册论坛事件，请检查自身是否拥有相关权限")

        if not on_forum_function:
            return wraps
        wraps(on_forum_function)

    def bind_open_forum(
        self, on_open_forum_function: Callable[[Model.OPEN_FORUMS], Any] = None
    ):
        """
        用作绑定接收公域论坛事件的函数

        .. note::
            当前仅可以接收FORUM_THREAD_CREATE、FORUM_THREAD_UPDATE、FORUM_THREAD_DELETE三个事件

        :param on_open_forum_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.OPEN_FORUMS, is_async=self.is_async)
            self._on_open_forum_function = func
            self._intents = self._intents | 1 << 18
            self.logger.info("论坛事件订阅成功")

        if not on_open_forum_function:
            return wraps
        wraps(on_open_forum_function)

    def bind_audio(self, on_audio_function: Callable[[Model.AUDIO_ACTION], Any] = None):
        """
        用作绑定接收论坛事件的函数

        :param on_audio_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.AUDIO_ACTION, is_async=self.is_async)
            self._on_audio_function = func
            self._intents = self._intents | 1 << 29
            self.logger.info("音频事件订阅成功")
            if self.no_permission_warning:
                self.logger.warning("请注意，一般机器人并不能注册音频事件（需先进行申请），请检查自身是否拥有相关权限")

        if not on_audio_function:
            return wraps
        wraps(on_audio_function)

    def bind_live_channel_member(
        self,
        on_live_channel_member_function: Callable[
            [Model.LIVE_CHANNEL_MEMBER], Any
        ] = None,
    ):
        """
        用作绑定接收音视频/直播子频道成员进出事件的函数

        :param on_live_channel_member_function: 类型为function，该函数应包含一个参数以接收Object消息数据进行处理
        """

        def wraps(func):
            check_func(func, Model.LIVE_CHANNEL_MEMBER, is_async=self.is_async)
            self._on_live_channel_member_function = func
            self._intents = self._intents | 1 << 19
            self.logger.info("音视频/直播子频道成员进出事件订阅成功")

        if not on_live_channel_member_function:
            return wraps
        wraps(on_live_channel_member_function)

    def register_repeat_event(
        self,
        time_function: Callable[[], Any] = None,
        check_interval: Union[float, int] = 10,
    ):
        """
        用作注册重复事件的函数，注册并开始机器人后，会根据间隔时间不断调用注册的函数

        :param time_function: 类型为function，该函数不应包含任何参数
        :param check_interval: 每多少秒检查调用一次时间事件函数，默认10
        """

        def wraps(func):
            check_func(func, is_async=self.is_async)
            self._repeat_function = func
            self.check_interval = check_interval
            self.logger.info("重复运行事件注册成功")

        if not time_function:
            return wraps
        wraps(time_function)

    def register_start_event(self, on_start_function: Callable[[], Any] = None):
        """
        用作注册机器人开始时运行的函数，此函数不应有无限重复的内容

        :param on_start_function: 类型为function，该函数不应包含任何参数
        """

        def wraps(func):
            check_func(func, is_async=self.is_async)
            self._on_start_function = func
            self.logger.info("初始事件注册成功")

        if not on_start_function:
            return wraps
        wraps(on_start_function)

    def security_setup(self, mini_id: str, mini_secret: str):
        """
        用于注册小程序ID和secret以使用腾讯内容安全接口

        :param mini_id: 小程序ID
        :param mini_secret: 小程序secret
        """
        self.api.security_setup(mini_id, mini_secret)

    def _check_warning(self, name: str):
        if not self.is_private and self.no_permission_warning:
            self.logger.warning(f"请注意，一般公域机器人并不能使用{name}API，请检查自身是否拥有相关权限")

    def start(self):
        """
        开始运行机器人的函数，在唤起此函数后的代码将不能运行，如需运行后续代码，请以多进程方式唤起此函数，以下是一个简单的唤起流程：

        >>> from qg_botsdk import BOT
        >>> bot = BOT(bot_id='xxx', bot_token='xxx')
        >>> bot.start()

        .. seealso::
            更多教程和相关资讯可参阅：
            https://qg-botsdk.readthedocs.io/zh_CN/latest/index.html
        """
        try:
            if not self.__running and not self._bot_class:
                self.__running = True
                gateway = self._loop.run_until_complete(
                    self._session.get(f"{self.bot_url}/gateway/bot")
                )
                gateway = self._loop.run_until_complete(gateway.json())
                url = gateway.get("url")
                if not url:
                    raise type("IdTokenError", (Exception,), {})(
                        "你输入的 bot_id 和/或 bot_token 错误，无法连接使用机器人\n如尚未有相关票据，"
                        "请参阅 https://qg-botsdk.readthedocs.io/zh_CN/latest/quick_start 了解相关详情"
                    )
                self.logger.debug("[机器人ws地址] " + url)
                commands, preprocessors = self._get_plugins()
                if self._repeat_function is not None:
                    self._loop.create_task(self.__time_event_check())
                self._bot_class = _BotWs(
                    self._session,
                    self.logger,
                    self._total_shard,
                    self._shard_no,
                    url,
                    self.auth,
                    self._on_msg_function,
                    self._on_dm_function,
                    self._on_delete_function,
                    self.is_filter_self,
                    self._on_guild_event_function,
                    self._on_channel_event_function,
                    self._on_guild_member_function,
                    self._on_reaction_function,
                    self._on_interaction_function,
                    self._on_audit_function,
                    self._on_forum_function,
                    self._on_open_forum_function,
                    self._on_audio_function,
                    self._on_live_channel_member_function,
                    self._intents,
                    self.msg_treat,
                    self.dm_treat,
                    self._on_start_function,
                    self.is_async,
                    self.max_workers,
                    self.api,
                    commands,
                    preprocessors,
                )
                self._bot_class.starter()
            else:
                self.logger.error("当前机器人已在运行中！")
        except KeyboardInterrupt:
            exit()

    def close(self):
        """
        结束运行机器人的函数

        .. seealso::
            更多教程和相关资讯可参阅：
            https://qg-botsdk.readthedocs.io/zh_CN/latest/index.html
        """
        if self.__running:
            self.__running = False
            self.logger.info("WS链接已开始结束进程，请等待另一端完成握手并等待 TCP 连接终止")
            self._bot_class.running = False
            timeout = self._loop.time() + 60
            while self._loop.time() < timeout:
                t_sleep(1)
                if not self._bot_class or self._bot_class.ws.closed:
                    return
            self._bot_class = None
            self.logger.info("判断超时，WS链接已强制结束")
        else:
            self.logger.error("当前机器人没有运行！")



from asyncio import sleep
from typing import Any, Awaitable, Callable


class Queue:
    def __init__(self, max_concurrency: int):
        self._running_queue = []
        self._MAX_RUNNING_SLOTS = max_concurrency

    async def create_task(
        self, task: Callable[[Any, Any], Awaitable[Any]], *args, **kwargs
    ):
        if self._MAX_RUNNING_SLOTS > 0:
            while not len(self._running_queue) < self._MAX_RUNNING_SLOTS:
                await sleep(0.5)
        self._running_queue.append(0)
        result = await task(*args, **kwargs)
        self._running_queue.pop()
        return result



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
from typing import List, Optional

from ._utils import object_class

apis = {
    ("获取用户ID", "get_bot_id"): [False, "此API不需要请求权限"],
    ("获取用户信息", "get_bot_info"): ["GET", "/users/@me"],
    ("获取用户频道列表", "get_bot_guilds"): ["GET", "/users/@me/guilds"],
    ("获取频道详情", "get_guild_info"): ["GET", "/guilds/{guild_id}"],
    ("获取子频道列表", "get_guild_channels"): ["GET", "/guilds/{guild_id}/channels"],
    ("获取子频道详情", "get_channels_info"): ["GET", "/channels/{channel_id}"],
    ("创建子频道", "create_channels"): ["POST", "/guilds/{guild_id}/channels"],
    ("修改子频道", "patch_channels"): ["PATCH", "/channels/{channel_id}"],
    ("删除子频道", "delete_channels"): ["DELETE", "/channels/{channel_id}"],
    ("获取频道成员列表", "get_guild_members"): ["GET", "/guilds/{guild_id}/members"],
    ("获取频道身份组成员列表", "get_role_members"): [
        "GET",
        "/guilds/{guild_id}/roles/{role_id}/members",
    ],
    ("获取频道成员详情", "get_member_info"): ["GET", "/guilds/{guild_id}/members/{user_id}"],
    ("删除频道成员", "delete_member"): ["DELETE", "/guilds/{guild_id}/members/{user_id}"],
    ("获取频道身份组列表", "get_guild_roles"): ["GET", "/guilds/{guild_id}/roles"],
    ("创建频道身份组", "create_role"): ["POST", "/guilds/{guild_id}/roles"],
    ("修改频道身份组", "patch_role"): ["PATCH", "/guilds/{guild_id}/roles/{role_id}"],
    ("删除频道身份组", "delete_role"): ["DELETE", "/guilds/{guild_id}/roles/{role_id}"],
    ("创建频道身份组成员", "create_role_member"): [
        "PUT",
        "/guilds/{guild_id}/members/{user_id}/roles/{role_id}",
    ],
    ("删除频道身份组成员", "delete_role_member"): [
        "DELETE",
        "/guilds/{guild_id}/members/{user_id}/roles/{role_id}",
    ],
    ("获取子频道用户权限", "get_channel_member_permission"): [
        "GET",
        "/channels/{channel_id}/members/{user_id}/permissions",
    ],
    ("修改子频道用户权限", "put_channel_member_permission"): [
        "PUT",
        "/channels/{channel_id}/members/{user_id}/permissions",
    ],
    ("获取子频道身份组权限", "get_channel_role_permission"): [
        "GET",
        "/channels/{channel_id}/roles/{role_id}/permissions",
    ],
    ("修改子频道身份组权限", "put_channel_role_permission"): [
        "PUT",
        "/channels/{channel_id}/roles/{role_id}/permissions",
    ],
    ("获取指定消息", "get_message_info"): [
        "GET",
        "/channels/{channel_id}/messages/{message_id}",
    ],
    (
        "发送普通消息",
        "send_msg",
        "发送embed模板消息",
        "send_embed",
        "发送 23 链接+文本列表模板ark消息",
        "send_ark_23",
        "发送 24 文本+缩略图模板ark消息",
        "send_ark_24",
        "发送 37 大图模板ark消息",
        "send_ark_37",
    ): ["POST", "/channels/{channel_id}/messages"],
    ("撤回消息", "delete_msg"): ["DELETE", "/channels/{channel_id}/messages/{message_id}"],
    ("获取频道消息频率设置", "get_guild_setting"): ["GET", "/guilds/{guild_id}/message/setting"],
    ("创建私信会话", "create_dm_guild"): ["POST", "/users/@me/dms"],
    ("发送私信消息", "send_dm"): ["POST", "/dms/{guild_id}/messages"],
    ("撤回私信消息", "delete_dm_msg"): ["DELETE", "/dms/{guild_id}/messages/{message_id}"],
    ("禁言全员", "mute_all_member"): ["PATCH", "/guilds/{guild_id}/mute"],
    ("禁言指定成员", "mute_member"): ["PATCH", "/guilds/{guild_id}/members/{user_id}/mute"],
    ("禁言批量成员", "mute_members"): ["PATCH", "/guilds/{guild_id}/mute"],
    ("创建频道公告", "create_announce"): ["POST", "/guilds/{guild_id}/announces"],
    ("删除频道公告", "delete_announce"): [
        "DELETE",
        "/guilds/{guild_id}/announces/{message_id}",
    ],
    ("添加精华消息", "create_pinmsg"): ["PUT", "/channels/{channel_id}/pins/{message_id}"],
    ("删除精华消息", "delete_pinmsg"): ["DELETE", "/channels/{channel_id}/pins/{message_id}"],
    ("获取精华消息", "get_pinmsg"): ["GET", "/channels/{channel_id}/pins"],
    ("获取频道日程列表", "get_schedules"): ["GET", "/channels/{channel_id}/schedules"],
    ("获取日程详情", "get_schedule_info"): [
        "GET",
        "/channels/{channel_id}/schedules/{schedule_id}",
    ],
    ("创建日程", "create_schedule"): ["POST", "/channels/{channel_id}/schedules"],
    ("修改日程", "patch_schedule"): [
        "PATCH",
        "/channels/{channel_id}/schedules/{schedule_id}",
    ],
    ("删除日程", "delete_schedule"): [
        "DELETE",
        "/channels/{channel_id}/schedules/{schedule_id}",
    ],
    ("发表表情表态", "create_reaction"): [
        "PUT",
        "/channels/{channel_id}/messages/{message_id}/reactions/{type}/{id}",
    ],
    ("删除表情表态", "delete_reaction"): [
        "DELETE",
        "/channels/{channel_id}/messages/{message_id}/reactions/{type}/{id}",
    ],
    ("拉取表情表态用户列表", "get_reaction_users"): [
        "GET",
        "/channels/{channel_id}/messages/{message_id}/reactions/{type}/{id}",
    ],
    ("音频控制", "control_audio"): ["POST", "/channels/{channel_id}/audio"],
    ("机器人上麦", "bot_on_mic"): ["PUT", "/channels/{channel_id}/mic"],
    ("机器人下麦", "bot_off_mic"): ["DELETE", "/channels/{channel_id}/mic"],
    ("获取帖子列表", "get_threads"): ["GET", "/channels/{channel_id}/threads"],
    ("获取帖子详情", "get_thread_info"): [
        "GET",
        "/channels/{channel_id}/threads/{thread_id}",
    ],
    ("发表帖子", "create_thread"): ["PUT", "/channels/{channel_id}/threads"],
    ("删除帖子", "delete_thread"): ["DELETE", "/channels/{channel_id}/threads/{thread_id}"],
    (
        "获取频道可用权限列表",
        "get_guild_permissions",
        "创建频道 API 接口权限授权链接",
        "create_permission_demand",
    ): [False, "此API不需要请求权限"],
}


def api_converter(api: str):
    if api[-2:] in ["()", "（）"]:
        api = api[:-2]
    for keys, values in apis.items():
        if api in keys:
            return values[1], values[0]
    return False, "该API并不存在"


def api_converter_re(method: str, path: str):
    for keys, values in apis.items():
        if [method, path] == values:
            return keys[1]


class _EmptyReturnTemplate:
    code: int
    message: str


def robot_model():
    class Robot:
        id: str
        username: str
        avatar: str

    return Robot


def get_bot_info():
    class GetBotInfo(object_class):
        class data:
            id: str
            username: str
            avatar: str
            union_openid: str
            union_user_account: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return GetBotInfo


def get_bot_guilds():
    class GetBotGuilds(object_class):
        class __guild:
            id: str
            name: str
            icon: str
            owner_id: str
            owner: bool
            joined_at: str
            member_count: str
            max_members: str
            description: str
            code: int
            message: str

        data: List[__guild]
        http_code: List[int]
        trace_id: List[str]
        result: List[bool]

    return GetBotGuilds


def get_guild_info():
    class GetGuildInfo(object_class):
        class data:
            id: str
            name: str
            icon: str
            owner_id: str
            owner: bool
            joined_at: str
            member_count: int
            max_members: int
            description: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return GetGuildInfo


def get_guild_channels():
    class GetGuildChannels(object_class):
        class __channels:
            id: str
            guild_id: str
            name: str
            type: int
            position: int
            parent_id: str
            owner_id: str
            sub_type: int
            private_type: int
            speak_permission: int
            application_id: str
            permissions: str
            code: int
            message: str

        data: List[__channels]
        http_code: int
        trace_id: str
        result: bool

    return GetGuildChannels


class _Channels:
    id: str
    guild_id: str
    name: str
    type: int
    position: int
    parent_id: str
    owner_id: str
    op_user_id: Optional[str]
    sub_type: int
    private_type: int
    private_user_ids: List[str]
    speak_permission: int
    application_id: str
    permissions: str
    code: int
    message: str


def get_channels_info():
    channels = _Channels

    class GetChannelsInfo(object_class):
        data: channels
        http_code: int
        trace_id: str
        result: bool

    return GetChannelsInfo


def create_channels():
    channels = _Channels

    class CreateChannels(object_class):
        data: channels
        http_code: int
        trace_id: str
        result: bool

    return CreateChannels


def patch_channels():
    channels = _Channels

    class PatchChannels(object_class):
        data: channels
        http_code: int
        trace_id: str
        result: bool

    return PatchChannels


def delete_channels():
    temp = _EmptyReturnTemplate

    class DeleteChannels(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteChannels


class _Member:
    class user:
        id: str
        username: str
        avatar: str
        bot: bool

    nick: str
    roles: List[str]
    joined_at: str
    deaf: bool
    mute: bool
    pending: bool
    code: int
    message: str


def get_guild_members():
    class GetGuildMembers(object_class):
        data: List[_Member]
        http_code: List[int]
        trace_id: List[str]
        result: List[bool]

    return GetGuildMembers


def get_role_members():
    class GetRoleMembers(object_class):
        data: List[_Member]
        http_code: List[int]
        trace_id: List[str]
        result: List[bool]

    return GetRoleMembers


def get_member_info():
    class GetMemberInfo(object_class):
        data: _Member
        http_code: int
        trace_id: str
        result: bool

    return GetMemberInfo


def delete_member():
    temp = _EmptyReturnTemplate

    class DeleteMember(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteMember


class _Role:
    id: str
    name: str
    color: int
    hoist: int
    number: int
    member_limit: int


def get_guild_roles():
    role_ = _Role

    class GetGuildRoles(object_class):
        class data:
            guild_id: str
            roles: List[role_]
            role_num_limit: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return GetGuildRoles


def create_role():
    role_ = _Role

    class CreateRole(object_class):
        class data:
            role_id: str
            role: role_
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return CreateRole


def patch_role():
    role_ = _Role

    class PatchRole(object_class):
        class data:
            guild_id: str
            role_id: str
            role: role_
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return PatchRole


def delete_role():
    temp = _EmptyReturnTemplate

    class DeleteRole(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteRole


def role_members():
    class RoleMembers(object_class):
        class data:
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return RoleMembers


class _Permission_Mem:
    channel_id: str
    user_id: str
    permissions: str
    code: int
    message: str


def get_channel_member_permission():
    permission = _Permission_Mem

    class GetChannelMemberPermission(object_class):
        data: permission
        http_code: int
        trace_id: str
        result: bool

    return GetChannelMemberPermission


def put_channel_mr_permission():
    temp = _EmptyReturnTemplate

    class PutChannelMRPermission(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return PutChannelMRPermission


class _Permission_Role:
    channel_id: str
    role_id: str
    permissions: str
    code: int
    message: str


def get_channel_role_permission():
    permission = _Permission_Role

    class GetChannelRolePermission(object_class):
        data: permission
        http_code: int
        trace_id: str
        result: bool

    return GetChannelRolePermission


class _Message:
    class __MsgAttachments:
        content_type: str
        filename: str
        height: int
        width: int
        id: str
        size: int
        url: str

    class message_reference:
        message_id: str

    class author:
        bot: bool
        id: str
        username: str

    class member:
        joined_at: str
        roles: List[str]

    channel_id: str
    guild_id: str
    content: str
    id: str
    attachments = List[__MsgAttachments]
    timestamp: str
    code: int
    message: str


def get_message_info():
    message_ = _Message

    class GetMessageInfo(object_class):
        class data:
            message: message_

        http_code: int
        trace_id: str
        result: bool

    return GetMessageInfo


def send_msg():
    class SendMsg(object_class):
        class data:
            id: str
            channel_id: str
            guild_id: str
            content: str
            timestamp: str
            tts: bool
            mention_everyone: bool
            author: dict
            pinned: bool
            type: int
            flags: int
            seq_in_channel: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return SendMsg


def delete_msg():
    temp = _EmptyReturnTemplate

    class DeleteMsg(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteMsg


def get_guild_setting():
    class GetGuildSetting(object_class):
        class data:
            disable_create_dm: bool
            disable_push_msg: bool
            channel_ids: List[str]
            channel_push_max_num: int
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return GetGuildSetting


def create_dm_guild():
    class CreateDmGuild(object_class):
        class data:
            guild_id: str
            channel_id: str
            create_time: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return CreateDmGuild


def mute_member():
    temp = _EmptyReturnTemplate

    class MuteMember(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return MuteMember


def mute_members():
    class MuteMembers(object_class):
        class data:
            user_ids: List[str]
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return MuteMembers


def create_announce():
    class RecommendChannels:
        channel_id: str
        introduce: str

    class CreateAnnounce(object_class):
        class data:
            guild_id: str
            channel_id: str
            message_id: str
            announces_type: int
            recommend_channels: List[RecommendChannels]
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return CreateAnnounce


def delete_announce():
    temp = _EmptyReturnTemplate

    class DeleteAnnounce(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteAnnounce


def pinmsg():
    class PinMsg(object_class):
        class data:
            guild_id: str
            channel_id: str
            message_ids: List[str]
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return PinMsg


def delete_pinmsg():
    temp = _EmptyReturnTemplate

    class DeletePinMsg(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeletePinMsg


class __Schedule:
    class creator:
        class user:
            id: str
            username: str
            bot: bool

        nick: str
        joined_at: str

    id: str
    name: str
    description: str
    start_timestamp: str
    end_timestamp: str
    jump_channel_id: str
    remind_type: str
    code: int
    message: str


def get_schedules():
    schedule = __Schedule

    class GetSchedules(object_class):
        data: List[schedule]
        http_code: int
        trace_id: str
        result: bool

    return GetSchedules


def schedule_info():
    schedule = __Schedule

    class ScheduleInfo(object_class):
        data: schedule
        http_code: int
        trace_id: str
        result: bool

    return ScheduleInfo


def delete_schedule():
    temp = _EmptyReturnTemplate

    class DeleteSchedule(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteSchedule


def reactions():
    temp = _EmptyReturnTemplate

    class Reactions(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return Reactions


def get_reaction_users():
    class Users:
        id: str
        username: str
        avatar: str
        code: int
        message: str

    class GetReactionUsers(object_class):
        data: List[Users]
        http_code: List[int]
        trace_id: List[str]
        result: List[bool]

    return GetReactionUsers


def audio():
    temp = _EmptyReturnTemplate

    class Audio(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return Audio


class __Threads:
    class thread_info:
        class content:
            class __ForumsContent:
                class __ForumsSubContent:
                    class text:  # type = 1
                        text: str

                    class image:  # type = 2
                        class plat_image:
                            url: str
                            width: int
                            height: int
                            image_id: str

                    class video:  # type = 3
                        class plat_video:
                            class cover:
                                url: str
                                width: int
                                height: int

                            url: str
                            width: int
                            height: int
                            video_id: str

                    class url:  # type = 4
                        url: str
                        desc: str

                    type = int

                elems: List[__ForumsSubContent]
                props: object

            paragraphs: List[__ForumsContent]

        thread_id: str
        title: str
        date_time: str

    guild_id: str
    channel_id: str
    author_id: str
    code: int
    message: str


def get_threads():
    thread = __Threads

    class GetThreads(object_class):
        data: List[thread]
        http_code: List[int]
        trace_id: List[str]
        result: List[bool]

    return GetThreads


def get_thread_info():
    thread = __Threads

    class GetThreadInfo(object_class):
        data: thread
        http_code: int
        trace_id: str
        result: bool

    return GetThreadInfo


def create_thread():
    class CreateThread(object_class):
        class data:
            task_id: str
            create_time: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return CreateThread


def delete_thread():
    temp = _EmptyReturnTemplate

    class DeleteThread(object_class):
        data: temp
        http_code: int
        trace_id: str
        result: bool

    return DeleteThread


def get_guild_permissions():
    class Api:
        api: str
        path: str
        method: str
        desc: str
        auth_status: int

    class GetGuildPermissions(object_class):
        class data:
            apis = List[Api]
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return GetGuildPermissions


def create_permission_demand():
    class CreatePermissionDemand(object_class):
        class data:
            class api_identify:
                path: str
                method: str

            guild_id: str
            channel_id: str
            title: str
            desc: str
            code: int
            message: str

        http_code: int
        trace_id: str
        result: bool

    return CreatePermissionDemand



class EVENTS:
    MESSAGE_CREATE = ("AT_MESSAGE_CREATE", "MESSAGE_CREATE")
    DM_CREATE = ("DIRECT_MESSAGE_CREATE",)
    MESSAGE_DELETE = (
        "MESSAGE_DELETE",
        "PUBLIC_MESSAGE_DELETE",
        "DIRECT_MESSAGE_DELETE",
    )
    FORUM = (
        "FORUM_THREAD_CREATE",
        "FORUM_THREAD_UPDATE",
        "FORUM_THREAD_DELETE",
        "FORUM_POST_CREATE",
        "FORUM_POST_DELETE",
        "FORUM_REPLY_CREATE",
        "FORUM_REPLY_DELETE",
        "FORUM_PUBLISH_AUDIT_RESULT",
    )
    GUILD = ("GUILD_CREATE", "GUILD_UPDATE", "GUILD_DELETE")
    CHANNEL = ("CHANNEL_CREATE", "CHANNEL_UPDATE", "CHANNEL_DELETE")
    GUILD_MEMBER = ("GUILD_MEMBER_ADD", "GUILD_MEMBER_UPDATE", "GUILD_MEMBER_REMOVE")
    REACTION = ("MESSAGE_REACTION_ADD", "MESSAGE_REACTION_REMOVE")
    INTERACTION = ("INTERACTION_CREATE",)
    AUDIT = ("MESSAGE_AUDIT_PASS", "MESSAGE_AUDIT_REJECT")
    OPEN_FORUM = (
        "OPEN_FORUM_THREAD_CREATE",
        "OPEN_FORUM_THREAD_UPDATE",
        "OPEN_FORUM_THREAD_DELETE",
        "OPEN_FORUM_POST_CREATE",
        "OPEN_FORUM_POST_DELETE",
        "OPEN_FORUM_REPLY_CREATE",
        "OPEN_FORUM_REPLY_DELETE",
    )
    AUDIO = ("AUDIO_START", "AUDIO_FINISH", "AUDIO_ON_MIC", "AUDIO_OFF_MIC")
    ALC_MEMBER = (
        "AUDIO_OR_LIVE_CHANNEL_MEMBER_ENTER",
        "AUDIO_OR_LIVE_CHANNEL_MEMBER_EXIT",
    )



from ssl import create_default_context
from typing import Optional

from aiohttp import (
    ClientSession,
    ClientTimeout,
    FormData,
    TCPConnector,
    hdrs,
    multipart,
    payload,
)

from ._queue import Queue
from ._utils import exception_handler, general_header, retry_err_code

try:
    from importlib.metadata import version

    aio_version = version("aiohttp")
except (ImportError, ModuleNotFoundError):
    from pkg_resources import get_distribution

    aio_version = get_distribution("aiohttp").version

version_checking = (3, 8, 1)
for version_index in range(3):
    try:
        if int(aio_version[2 * version_index]) < version_checking[version_index]:
            print(
                f"\033[1;33m[warning] 注意你的aiohttp版本为{aio_version}，SDK建议升级到3.8.1，避免出现无法预计的错误\033[0m"
            )
            break
    except (ValueError, IndexError):
        pass


# derived from aiohttp FormData object, changing the return of _is_processed to allow retry using the same data object
class FormData_(FormData):
    def _gen_form_data(self) -> multipart.MultipartWriter:
        """Encode a list of fields using the multipart/form-data MIME format"""
        if self._is_processed:
            return self._writer
        for dispparams, headers, value in self._fields:
            try:
                if hdrs.CONTENT_TYPE in headers:
                    part = payload.get_payload(
                        value,
                        content_type=headers[hdrs.CONTENT_TYPE],
                        headers=headers,
                        encoding=self._charset,
                    )
                else:
                    part = payload.get_payload(
                        value, headers=headers, encoding=self._charset
                    )
            except Exception as exc:
                print(value)
                raise TypeError(
                    "Can not serialize value type: %r\n "
                    "headers: %r\n value: %r" % (type(value), headers, value)
                ) from exc

            if dispparams:
                part.set_content_disposition(
                    "form-data", quote_fields=self._quote_fields, **dispparams
                )
                assert part.headers is not None
                part.headers.popall(hdrs.CONTENT_LENGTH, None)

            self._writer.append_payload(part)

        self._is_processed = True
        return self._writer


class Session:
    def __init__(self, loop, is_retry, is_log_error, logger, max_concurrency, **kwargs):
        self._is_retry = is_retry
        self._is_log_error = is_log_error
        self._logger = logger
        self._queue = Queue(max_concurrency)
        self._kwargs = kwargs
        if kwargs.get("connector") is None:
            kwargs["connector"] = TCPConnector(
                limit=0, ssl=create_default_context(), force_close=True
            )
        self._session: Optional[ClientSession] = None
        self._timeout = ClientTimeout(total=20)
        self._loop = loop
        loop.run_until_complete(self._check_session())

    def __del__(self):
        if self._session and not self._session.closed:
            self._loop.run_until_complete(self._session.close())

    async def _check_session(self):
        if not self._session or self._session.closed:
            self._session = ClientSession(timeout=self._timeout, **self._kwargs)

    async def _warning(self, url, resp):
        self._logger.warning(
            f"HTTP API(url:{url})调用错误[{resp.status}]，详情：{await resp.text()}，"
            f'trace_id：{resp.headers.get("X-Tps-Trace-Id", None)}'
        )

    def __getattr__(self, item):
        if item in ("get", "post", "delete", "patch", "put"):

            def wrap(*args, **kwargs):
                try:
                    return self._queue.create_task(self._request, item, *args, **kwargs)
                except Exception as e:
                    self._logger.error(
                        f"HTTP API(url:{args[0]})调用错误，详情：{exception_handler(e)}"
                    )

            return wrap

    async def _request(self, method, url, retry=False, **kwargs):
        await self._check_session()
        kwargs["headers"] = kwargs.get("headers", general_header)
        resp = await self._session.request(method, url, **kwargs)
        if resp.ok:
            return resp
        if self._is_log_error and (not self._is_retry or retry):
            await self._warning(url, resp)
        if self._is_retry and not retry:
            if resp.headers["content-type"] == "application/json":
                json_ = await resp.json()
                if (
                    not isinstance(json_, dict)
                    or json_.get("code", None) not in retry_err_code
                ):
                    await self._warning(url, resp)
                    return resp
            return await self.request(method, url, True, **kwargs)
        return resp



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
from typing import List, Union

from ._utils import event_class


class Model:
    """
    使用此Model库可用作验证事件数据的准确性，目前可用的模型如下：

    GUILDS - 频道事件
    GUILD_MEMBERS -
    """

    class GUILDS(event_class):
        """
        频道事件的数据模型，可从t字段判断具体事件，其中包含：

        - GUILD_CREATE - 当机器人加入新guild时
        - GUILD_UPDATE - 当guild资料发生变更时
        - GUILD_DELETE - 当机器人退出guild时

        .. seealso::
            其子字段数据可参阅：
            https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#guilds
        """

        description: str
        icon: str
        id: str
        joined_at: str
        max_members: int
        member_count: int
        name: str
        op_user_id: str
        owner_id: str
        t: str
        event_id: str

    class CHANNELS(event_class):
        """
        子频道事件的数据模型，可从t字段判断具体事件，其中包含：

        - CHANNEL_CREATE - 当channel被创建时
        - CHANNEL_UPDATE - 当channel被更新时
        - CHANNEL_DELETE - 当channel被删除时

        .. seealso::
            其子字段数据可参阅：
            https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#channels-sdkv2-2-4
        """

        application_id: str
        guild_id: str
        id: str
        name: str
        op_user_id: str
        owner_id: str
        parent_id: str
        permissions: str
        position: int
        private_type: int
        speak_permission: int
        sub_type: int
        type: int
        t: str
        event_id: str

    class GUILD_MEMBERS(event_class):
        """
        频道成员事件的数据模型，可从t字段判断具体事件，其中包含：

        - GUILD_MEMBER_ADD - 当成员加入时
        - GUILD_MEMBER_UPDATE - 当成员资料变更时
        - GUILD_MEMBER_REMOVE - 当成员被移除时

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#guild-members
        """

        class user:
            avatar: str
            bot: bool
            id: str
            username: str

        guild_id: str
        joined_at: str
        nick: str
        op_user_id: str
        roles: List[str]
        t: str
        event_id: str

    class MESSAGE(event_class):
        """
        消息事件的数据模，可从t字段判断具体事件，其中包含：

        - MESSAGE_CREATE - 发送消息事件，代表频道内的全部消息，而不只是 at 机器人的消息
        - AT_MESSAGE_CREATE - 当收到@机器人的消息时

        .. note::
            私域机器人可接收全部消息事件、公域机器人仅能接收艾特机器人的消息事件

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#message
        """

        class __MsgMentions:
            avatar: str
            bot: bool
            id: str
            username: str

        class __MsgAttachments:
            content_type: str
            filename: str
            height: int
            width: int
            id: str
            size: int
            url: str

        class message_reference:
            message_id: str

        class author:
            avatar: str
            bot: bool
            id: str
            username: str

        class member:
            joined_at: str
            nick: str
            roles: List[str]

        channel_id: str
        guild_id: str
        content: str
        id: str
        mentions = List[__MsgMentions]
        attachments = List[__MsgAttachments]
        seq: int
        seq_in_channel: str
        timestamp: str
        tts: bool
        pinned: bool
        type: int
        flags: int
        treated_msg: Union[str, tuple]
        t: str
        event_id: str

    class MESSAGE_DELETE(event_class):
        """
        消息撤回事件的数据模型，可从t字段判断具体事件，其中包含：

        - MESSAGE_DELETE - 删除（撤回）消息事件
        - PUBLIC_MESSAGE_DELETE - 当频道的消息被删除时
        - DIRECT_MESSAGE_DELETE - 删除（撤回）私信消息事件

        .. note::
            私域机器人可接收MESSAGE_DELETE、公域机器人仅能接收PUBLIC_MESSAGE_DELETE

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#message-delete
        """

        class message:
            class author:
                bot: bool
                id: str
                username: str

            channel_id: str
            guild_id: str
            id: str

        class op_user:
            id: str

        t: str
        event_id: str

    class DIRECT_MESSAGE(event_class):
        """
        私信消息事件的数据模型，可从t字段判断具体事件，其中包含：

        - DIRECT_MESSAGE_CREATE - 当收到用户发给机器人的私信消息时

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#direct-message
        """

        class __MsgAttachments:
            content_type: str
            filename: str
            height: int
            width: int
            id: str
            size: int
            url: str

        class message_reference:
            message_id: str

        class author:
            avatar: str
            id: str
            username: str

        class member:
            joined_at: str

        channel_id: str
        guild_id: str
        content: str
        direct_message: bool
        id: str
        attachments = List[__MsgAttachments]
        seq: int
        seq_in_channel: str
        src_guild_id: str
        timestamp: str
        treated_msg: str
        t: str
        event_id: str

    class MESSAGE_AUDIT(event_class):
        """
        主动消息审核事件的数据模型，可从t字段判断具体事件，其中包含：

        - MESSAGE_AUDIT_PASS - 消息审核通过
        - MESSAGE_AUDIT_REJECT - 消息审核不通过

        .. note::
            注意，只有审核通过事件（MESSAGE_AUDIT_PASS）才会有message_id的值

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#message-audit
        """

        audit_id: str
        audit_time: str
        create_time: str
        channel_id: str
        guild_id: str
        message_id: str
        t: str
        event_id: str

    class FORUMS_EVENT(event_class):
        """
        论坛事件的数据模型，可从t字段判断具体事件，其中包含：

        - FORUM_THREAD_CREATE - 当用户创建主题（帖子）时
        - FORUM_THREAD_UPDATE - 当用户更新主题（帖子）时
        - FORUM_THREAD_DELETE - 当用户删除主题（帖子）时

        *剩余论坛事件（如回帖和回复回帖）暂未有相关推送*

        .. note::
            现有推送的type字段：

            - type 1 - 普通文本，子字段text
            - type 4 - url信息，子字段url

            现无推送，根据文档列出的type：

            - 原新type 2 - 图片，子字段image（曾短暂出现，目前为空子字段，无任何内容反馈）
            - 原新type 3 - 视频，子字段video（曾短暂出现，目前为空子字段，无任何内容反馈）
            - 原type 2 - at信息，目前为空子字段，无任何内容反馈
            - 原type 4 - 表情，目前为空子字段，无任何内容反馈
            - 原type 5 - #子频道，目前为空子字段，无任何内容反馈

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#forums-event
        """

        class thread_info:
            class title:
                class __ForumsTitle:
                    class __ForumsSubTitle:
                        class text:
                            text: str

                        type: int

                    elems: List[__ForumsSubTitle]
                    props: object

                paragraphs: List[__ForumsTitle]

            class content:
                class __ForumsContent:
                    class __ForumsSubContent:
                        class text:  # type = 1
                            text: str

                        #                         class image:  # type = 2
                        #                             class plat_image:
                        #                                 url: str
                        #                                 width: int
                        #                                 height: int
                        #                                 image_id: str
                        #
                        #                         class video:  # type = 3
                        #                             class plat_video:
                        #                                 class cover:
                        #                                     url: str
                        #                                     width: int
                        #                                     height: int
                        #
                        #                                 url: str
                        #                                 width: int
                        #                                 height: int
                        #                                 video_id: str

                        class url:  # type = 4
                            url: str
                            desc: str

                        type = int

                    elems: List[__ForumsSubContent]
                    props: object

                paragraphs: List[__ForumsContent]

            thread_id: str
            date_time: str

        guild_id: str
        channel_id: str
        author_id: str
        t: str
        event_id: str

    class OPEN_FORUMS(event_class):
        """
        公域论坛事件的数据模型，可从t字段判断具体事件，其中包含：

        - OPEN_FORUM_THREAD_CREATE - 当用户创建主题（帖子）时
        - OPEN_FORUM_THREAD_UPDATE - 当用户更新主题（帖子）时
        - OPEN_FORUM_THREAD_DELETE - 当用户删除主题（帖子）时

        *剩余公域论坛事件（如回帖和回复回帖）暂未有相关推送*

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#open-forums
        """

        guild_id: str
        channel_id: str
        author_id: str
        t: str
        event_id: str

    class AUDIO_ACTION(event_class):
        """
        音频事件的数据模型，可从t字段判断具体事件，其中包含：

        - AUDIO_START - 音频开始播放时
        - AUDIO_FINISH - 音频播放结束时
        - AUDIO_ON_MIC - 上麦时
        - AUDIO_OFF_MIC - 下麦时

        .. note::
            注意，只有AUDIO_START、AUDIO_FINISH拥有audio_url和text字段

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#audio-action
        """

        channel_id: str
        guild_id: str
        audio_url: str
        text: str
        t: str
        event_id: str

    class REACTION(event_class):
        """
        表情表态事件的数据模型，可从t字段判断具体事件，其中包含：

        - MESSAGE_REACTION_ADD - 为消息添加表情表态
        - MESSAGE_REACTION_REMOVE - 为消息删除表情表态

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#reaction
        """

        class emoji:
            id: str
            type: int

        class target:
            id: str
            type: int

        user_id: str
        channel_id: str
        guild_id: str
        t: str
        event_id: str

    class INTERACTION(event_class):
        """
        互动事件的数据模型，可从t字段判断具体事件，其中包含：

        - INTERACTION_CREATE - 互动事件创建时

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#interaction
        """

        class data:
            class resolved:
                button_data: str
                button_id: str
                message_id: str
                user_id: str

            type: int

        application_id: str
        channel_id: str
        guild_id: str
        id: str
        type: int
        version: int
        t: str
        event_id: str

    class LIVE_CHANNEL_MEMBER(event_class):
        """
        音视频/直播子频道成员进出事件的数据模型，可从t字段判断具体事件，其中包含：

        - AUDIO_OR_LIVE_CHANNEL_MEMBER_ENTER - 当用户进入音视频/直播子频道
        - AUDIO_OR_LIVE_CHANNEL_MEMBER_EXIT - 当用户离开音视频/直播子频道

        .. seealso::
             其子字段数据可参阅：
             https://qg-botsdk.readthedocs.io/zh_CN/latest/Model%E5%BA%93.html#live_channel_member
        """

        channel_id: str
        channel_type: int
        guild_id: str
        user_id: str
        t: str
        event_id: str


class EmojiID:
    得意 = 4
    流泪 = 5
    睡 = 8
    大哭 = 9
    尴尬 = 10
    调皮 = 12
    微笑 = 14
    酷 = 16
    可爱 = 21
    傲慢 = 23
    饥饿 = 24
    困 = 25
    惊恐 = 26
    流汗 = 27
    憨笑 = 28
    悠闲 = 29
    奋斗 = 30
    疑问 = 32
    嘘 = 33
    晕 = 34
    敲打 = 38
    再见 = 39
    发抖 = 41
    爱情 = 42
    跳跳 = 43
    拥抱 = 49
    蛋糕 = 53
    咖啡 = 60
    玫瑰 = 63
    爱心 = 66
    太阳 = 74
    月亮 = 75
    赞 = 76
    握手 = 78
    胜利 = 79
    飞吻 = 85
    西瓜 = 89
    冷汗 = 96
    擦汗 = 97
    抠鼻 = 98
    鼓掌 = 99
    糗大了 = 100
    坏笑 = 101
    左哼哼 = 102
    右哼哼 = 103
    哈欠 = 104
    委屈 = 106
    左亲亲 = 109
    可怜 = 111
    示爱 = 116
    抱拳 = 118
    拳头 = 120
    爱你 = 122
    NO = 123
    OK = 124
    转圈 = 125
    挥手 = 129
    喝彩 = 144
    棒棒糖 = 147
    茶 = 171
    泪奔 = 173
    无奈 = 174
    卖萌 = 175
    小纠结 = 176
    doge = 179
    惊喜 = 180
    骚扰 = 181
    笑哭 = 182
    我最美 = 183
    点赞 = 201
    托脸 = 203
    托腮 = 212
    啵啵 = 214
    蹭一蹭 = 219
    抱抱 = 222
    拍手 = 227
    佛系 = 232
    喷脸 = 240
    甩头 = 243
    加油抱抱 = 246
    脑阔疼 = 262
    捂脸 = 264
    辣眼睛 = 265
    哦哟 = 266
    头秃 = 267
    问号脸 = 268
    暗中观察 = 269
    emm = 270
    吃瓜 = 271
    呵呵哒 = 272
    我酸了 = 273
    汪汪 = 277
    汗 = 278
    无眼笑 = 281
    敬礼 = 282
    面无表情 = 284
    摸鱼 = 285
    哦 = 287
    睁眼 = 289
    敲开心 = 290
    摸锦鲤 = 293
    期待 = 294
    拜谢 = 297
    元宝 = 298
    牛啊 = 299
    右亲亲 = 305
    牛气冲天 = 306
    喵喵 = 307
    仔细分析 = 314
    加油 = 315
    崇拜 = 318
    比心 = 319
    庆祝 = 320
    拒绝 = 322
    吃糖 = 324
    生气 = 326


class EmojiString:
    得意 = "<emoji:4>"
    流泪 = "<emoji:5>"
    睡 = "<emoji:8>"
    大哭 = "<emoji:9>"
    尴尬 = "<emoji:10>"
    调皮 = "<emoji:12>"
    微笑 = "<emoji:14>"
    酷 = "<emoji:16>"
    可爱 = "<emoji:21>"
    傲慢 = "<emoji:23>"
    饥饿 = "<emoji:24>"
    困 = "<emoji:25>"
    惊恐 = "<emoji:26>"
    流汗 = "<emoji:27>"
    憨笑 = "<emoji:28>"
    悠闲 = "<emoji:29>"
    奋斗 = "<emoji:30>"
    疑问 = "<emoji:32>"
    嘘 = "<emoji:33>"
    晕 = "<emoji:34>"
    敲打 = "<emoji:38>"
    再见 = "<emoji:39>"
    发抖 = "<emoji:41>"
    爱情 = "<emoji:42>"
    跳跳 = "<emoji:43>"
    拥抱 = "<emoji:49>"
    蛋糕 = "<emoji:53>"
    咖啡 = "<emoji:60>"
    玫瑰 = "<emoji:63>"
    爱心 = "<emoji:66>"
    太阳 = "<emoji:74>"
    月亮 = "<emoji:75>"
    赞 = "<emoji:76>"
    握手 = "<emoji:78>"
    胜利 = "<emoji:79>"
    飞吻 = "<emoji:85>"
    西瓜 = "<emoji:89>"
    冷汗 = "<emoji:96>"
    擦汗 = "<emoji:97>"
    抠鼻 = "<emoji:98>"
    鼓掌 = "<emoji:99>"
    糗大了 = "<emoji:100>"
    坏笑 = "<emoji:101>"
    左哼哼 = "<emoji:102>"
    右哼哼 = "<emoji:103>"
    哈欠 = "<emoji:104>"
    委屈 = "<emoji:106>"
    左亲亲 = "<emoji:109>"
    可怜 = "<emoji:111>"
    示爱 = "<emoji:116>"
    抱拳 = "<emoji:118>"
    拳头 = "<emoji:120>"
    爱你 = "<emoji:122>"
    NO = "<emoji:123>"
    OK = "<emoji:124>"
    转圈 = "<emoji:125>"
    挥手 = "<emoji:129>"
    喝彩 = "<emoji:144>"
    棒棒糖 = "<emoji:147>"
    茶 = "<emoji:171>"
    泪奔 = "<emoji:173>"
    无奈 = "<emoji:174>"
    卖萌 = "<emoji:175>"
    小纠结 = "<emoji:176>"
    doge = "<emoji:179>"
    惊喜 = "<emoji:180>"
    骚扰 = "<emoji:181>"
    笑哭 = "<emoji:182>"
    我最美 = "<emoji:183>"
    点赞 = "<emoji:201>"
    托脸 = "<emoji:203>"
    托腮 = "<emoji:212>"
    啵啵 = "<emoji:214>"
    蹭一蹭 = "<emoji:219>"
    抱抱 = "<emoji:222>"
    拍手 = "<emoji:227>"
    佛系 = "<emoji:232>"
    喷脸 = "<emoji:240>"
    甩头 = "<emoji:243>"
    加油抱抱 = "<emoji:246>"
    脑阔疼 = "<emoji:262>"
    捂脸 = "<emoji:264>"
    辣眼睛 = "<emoji:265>"
    哦哟 = "<emoji:266>"
    头秃 = "<emoji:267>"
    问号脸 = "<emoji:268>"
    暗中观察 = "<emoji:269>"
    emm = "<emoji:270>"
    吃瓜 = "<emoji:271>"
    呵呵哒 = "<emoji:272>"
    我酸了 = "<emoji:273>"
    汪汪 = "<emoji:277>"
    汗 = "<emoji:278>"
    无眼笑 = "<emoji:281>"
    敬礼 = "<emoji:282>"
    面无表情 = "<emoji:284>"
    摸鱼 = "<emoji:285>"
    哦 = "<emoji:287>"
    睁眼 = "<emoji:289>"
    敲开心 = "<emoji:290>"
    摸锦鲤 = "<emoji:293>"
    期待 = "<emoji:294>"
    拜谢 = "<emoji:297>"
    元宝 = "<emoji:298>"
    牛啊 = "<emoji:299>"
    右亲亲 = "<emoji:305>"
    牛气冲天 = "<emoji:306>"
    喵喵 = "<emoji:307>"
    仔细分析 = "<emoji:314>"
    加油 = "<emoji:315>"
    崇拜 = "<emoji:318>"
    比心 = "<emoji:319>"
    庆祝 = "<emoji:320>"
    拒绝 = "<emoji:322>"
    吃糖 = "<emoji:324>"
    生气 = "<emoji:326>"
